var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-9F96FB7D-EB51-41FA-976D-C39DEED69CE4\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Global and Private Custom Attributes Definitions</h1></div><p>Scripted custom attributes are properly saved to and loaded from scene files.</p>\n\
<p>You can assign a fixed, global definition ID to an attributes definition.</p>\n\
<p>This acts in a similar way to class IDs in scripted plug-ins. Such definitions are <strong>global</strong> across any number of scenes that custom attributes may be present in and all instances of custom attributes of a global definition will have the same shape. The ID is specified with a new header parameter, &apos; <code>attribID:</code> &#x2019;</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   attributes &lt;name&gt; attribID: #(&lt;number&gt;, &lt;number&gt;) ( ... )                   </code></pre></blockquote>\n\
<p>The ID number pair is chosen to be unique, perhaps generated with the <a href=\"./../../MAXScript-Tools-and-Interaction/Creating-MAXScript-Tools/Scripted-Plug-ins/The-Super-Class-ID-and-the-Class.html\">genClassID</a>() function in MAXScript. Attribute definitions without attribID&apos;s are private to the current MAX session.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">genClassID [     returnValue:&lt;boolean&gt;]   </code></pre><p>This method generates a random class ID similar to #(0x9b7ea231, 0xb6db86ef), and prints it to Listener. You can just cut and paste this class ID into your script to use the generated ID. If the optional returnValue keyword is supplied and set to true, the value will be returned as the result of the method call.</p>\n\
<p>Definitions are identified depending on whether they are global or private definitions.</p>\n\
<p><strong>Global definitions</strong> have explicit attribID: parameters, which define a globally unique ID for that definition and ensure that any object with custom attributes based on a definition with that ID will be follow the same definition. Whenever you evaluate an attributes definition with a particular attribID: you will update all objects that have custom attributes based on that definition, and loading objects with attributes of that attribID will update themselves to the current definition.</p>\n\
<p><strong>Private definitions</strong> have no explicit attribID: parameter (they effectively have an internally-generated random one). Each time you evaluate one, except as described in the redefinition section below, a completely distinct definition is created with its own internal ID.</p>\n\
<p>In the case of both global and private definitions, the &apos;attributes&apos; definition construct returns the definition as its value, so you would typically assign that value to a local and then use it to add custom attributes to objects via the <code>custAttributes.add ()</code> function.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   local def\n\
   def = attributes &quot;Game Data&quot;\n\
   (\n\
   parameters ...\n\
   rollout ...\n\
   )\n\
   custAttributes.add $box01 def\n\
   custAttributes.add $box02 def\n\
</code></pre></blockquote>\n\
<p>This example adds a separate set of the defined custom attributes to box01 and box02. The definition is shared between the two boxes.</p>\n\
<p>This is a private definition, it has no attribID:, so if you evaluated the &apos;def = attributes (...)&apos; again, it would <code>\\*not\\*</code> automatically update box01&apos;s and box02&apos;s attributes, as used to happen in prior builds, but create a new definition.</p>\n\
<h3 id=\"redefining-custom-attributes\">Redefining Custom Attributes</h3><p>In order to redefine a specific attribute definition so that all objects that had attributes added using it will be updated, you can use one of two schemes that operate on the actual definition value (as it sits in the &apos;def&apos; variable in the above, for example).</p>\n\
<p>First, you can use the &apos;redefine:&apos; attributes definition parameter:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   attributes &quot;Game Data&quot;\n\
   redefine:def\n\
   (\n\
   parameters ...\n\
   rollout ...\n\
   )\n\
</code></pre></blockquote>\n\
<p>This expects an existing attributes definition as the value given in the redefine: parameter and will update it to the definition that follows, and update any custom attributes made from that particular definition value. In the example, this would update $box01 and $box02.</p>\n\
<p>Alternatively, you can use the <code>custAttributes.redefine</code> method:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">custAttributes.redefine &lt;def&gt; &lt;definition_string&gt;   </code></pre><blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   local new_def = &quot;attributes \\&quot;Game Data\\&quot; ( ... )&quot;\n\
   custAttributes.redefine def new_def\n\
</code></pre></blockquote>\n\
<p>This is particularly useful in situations where you are generating the attributes definition automatically as a string. You can use this function to compile and apply the redefinition to a particular definition object in one go, without having to construct a name for the object, as you would if you used the redefine: parameter scheme.</p>\n\
<p>To redefine the attributes on some object, say based on the defData in that object&apos;s custom attributes, you might use this sequence:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">old_def = custAttributes.getDef $box03 1 -- get existing def\n\
old_def_data = custAttributes.getDefData old_def -- get my defData from it\n\
new_def_string = generate_new_def old_def_data ... -- make new def string\n\
custAttributes.redefine old_def new_def_string -- redefine it</code></pre><p>These redefinition techniques can be used with global definitions (with explicit attribID:), but are strictly unnecessary since the particular definition value is specified uniquely by the attribID: parameter. In any situation that you evaluate a global definition, all custom attributes in the current scene made from that definition will be updated.</p>\n\
<p><strong>Next Topic:</strong></p>\n\
<p><a href=\"Custom-Attribute-Management.html\">Custom Attribute Management Functions</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Custom-Attributes.html\">Custom Attributes</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Scripted-Custom-Attributes.html\">Scripted Custom Attributes</a></li>\n\
          <li><a class=\"link\" href=\"Adding-Custom-Attributes.html\">Adding Custom Attributes</a></li>\n\
          <li><a class=\"link\" href=\"Custom-Attributes-Definition.html\">Custom Attributes Definition Values</a></li>\n\
          <li><a class=\"link\" href=\"Access-to-Custom-Attributes.html\">Access to Custom Attributes Virtual Array</a></li>\n\
          <li><a class=\"link\" href=\"Custom-Attributes-in-Materials.html\">Custom Attributes in Materials and Texture Maps</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";