var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-BB29A275-58EF-4686-BA58-B9B6265B1B39\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Interfaces</h1></div><p>The Function Publishing System, introduced with <strong>3ds Max version 4</strong>, is a system that allows plug-ins to publish their major functions and operations in such a way that code outside the plug-in can discover and make inquiries about these functions and is thus able to call them though a common calling mechanism. The whole system is very similar to Window&#x2019;s COM and OLE Automation systems and share many similar concepts in the architecture. However, the Function Publishing System is not based on COM and OLE but instead is a custom architecture more suited and optimized for <strong>3ds Max</strong>. The Function Publishing API serves a number of purposes, which allow third-party developers to open up important portions of their plug-ins for use by external sources, allowing for users to extend and control these directly.</p>\n\
<p>Some of the purposes of the Function Publishing System are:</p>\n\
<ul>\n\
<li><p>Modularizing plugin code into various &quot;engines&quot; that are able to supply services to other parts of 3ds Max and other third party plug-ins and can be delivered to the user through various different user-interfaces.</p>\n\
</li>\n\
<li><p>Providing automatic scriptability by exposing the published functions directly to MAXScript.</p>\n\
</li>\n\
<li><p>Providing alternate means of invoking plugin functions in the UI, such as via the manipulator system, scripted menus, quad menus, keyboard shortcuts, macroScripts, toolbar buttons, etc.</p>\n\
</li>\n\
<li><p>Allowing the MAXScript Macro Recorder to automatically generate calls to the published functions, in the event that these are invoked by the expanded ParamMap2 system or other UI mechanisms such as keyboard shortcuts or menu items.</p>\n\
</li>\n\
</ul>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> For complete details on the Function Publishing System, please see the <strong>3ds Max</strong> SDK help file topic &quot;Function Publishing&quot;.</div>\n\
<p><strong>Interfaces</strong></p>\n\
<p><a href=\"Core-Interfaces/Core-Interfaces-1.html\">Core Interfaces</a></p>\n\
<p><a href=\"Node-Interfaces/Node-Interfaces-1.html\">Node Interfaces</a></p>\n\
<p><a href=\"Other-Interfaces/Other-Interfaces-1.html\">Other Interfaces</a></p>\n\
<p>Functions are published in one or more Interfaces by a plug-in. Action functions must be published in their own set of interfaces. Each interface is represented by an instance of a class derived from the base class FPInterface. An external system can find out about the interfaces published by calling various query methods on ClassDesc that access these interface definition objects. As well as these inquiry or &apos;reflection&apos; methods, an FPInterface also has the calling methods for actually invoking a particular function in the interface, so that if something has hold of one of your interfaces, it can call <strong>any</strong> of its published functions.</p>\n\
<p><strong>Action Interfaces</strong></p>\n\
<p><a href=\"./../../MAXScript-Tools-and-Interaction/Interacting-with-the-3ds-Max/Action-Manager-and-Shortcut/Action-Manager.html\">Action Manager</a></p>\n\
<p>A special kind of interface is the Action Interface. These interfaces only contain UI Action Functions that provide a programmatic way of &quot;pressing&quot; buttons and keys in the UI for a plug-in.</p>\n\
<p><strong>Direct dot-notation properties</strong></p>\n\
<p>MAXScript exposes properties defined in this way as direct dot-notation properties on the interface. So, were before the Get/SetRadius in the example cylinder mixin would have been accessed as functions</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   r = $cyl01.cylMixin.getRadius()\n\
   $cyl01.cylMixin.setRadius 23\n\
</code></pre><p>   you now can use:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   r = $cyl01.cylMixin.radius\n\
   $cyl01.cylMixin.radius = 23\n\
</code></pre></blockquote>\n\
<p>This is true for all the types of Function Published Interfaces that exist in MAXScript-static, <a href=\"./../../Frequently-Asked-Questions/MAXScript-Syntax-and-Terminology/What-is-a-Mixin-Interface-.html\">Mixin</a> and Core. As a further optimization, MAXScript effectively promotes all interface methods and properties to the level of the interface, so if individual methods and properties have unique names within all the interfaces of an object or class, you can omit the interface name.</p>\n\
<p>The above examples could be written as</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   r = $cyl01.getRadius()\n\
   $cyl01.setRadius 23\n\
</code></pre><p>   and</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   r = $cyl01.radius\n\
   $cyl01.radius = 23\n\
</code></pre></blockquote>\n\
<p>If there is a naming conflict, you can always include the interface name level to resolve this.</p>\n\
<h2 id=\"published-functions-and-maxscript\">Published Functions and MAXScript</h2><p>MAXScript automatically provides access to all functions published by a plug-in via the Function Publishing system. Each plug-in class appears in MAXScript as a MAX class object, which can be used to construct instances of the plug-in, do class tests, etc. If a plug-in publishes interfaces, they are visible in MAXScript as properties on this class object. The internal name for the interface is used as the property name. All the functions in the interface are accessible as named properties on the interface. So, if the above example interfaces were published by EditMesh, the following script fragments would work</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   EditMesh.faceOps.extrude $foo.mesh #{1,2,3} 10\n\
</code></pre></blockquote>\n\
<p>The above script calls the Extrude function in the FaceOps interface on <code>$foo</code> &apos;s mesh, faces 1, 2 and 3, amount 10 units.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">EditMesh.actions</code></pre><p>Retrieves and displays the action functions. Each interface is stored as a struct definition in the class object.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">EditMesh.actions.create()</code></pre><p>Starts (or stops) the create mode. This would have the side-effect of highlighting/unhighlighting the Create button in the EditMesh rollups. Calls to Action functions in MAXScript return true if the function returns FPS_OK and false otherwise.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">if EditMesh.actions.create.isChecked() then ...\n\
</code></pre><p>The predicate functions for an Action Function are available as properties on the action function object itself, as shown. You can determine if a predicate is supplied by asking:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">if EditMesh.actions.create.isChecked != undefined</code></pre><p><strong>Two Ways of Accessing the Interface</strong></p>\n\
<p>There are two ways to access the interface - as a standalone interface and as an interface on a particular object. If you access the interface directly, you need to pass to it the object to work on.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   m = Bitmaptexture fileName:&quot;L:\\\\mypath\\\\scenes\\\\2500hicon.JPG&quot; \n\
   bitmapTex.viewImage m -- direct interface access, need to pass object\n\
   m.viewImage() -- accessing interface on object, object internally passed as arg 1.\n\
</code></pre></blockquote>\n\
<p><strong>Associated Method:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getCoreInterfaces()</code></pre><p>Returns an array of core interface values.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<p>   The following will list all of the core interfaces:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   core_interfaces = getcoreinterfaces()\n\
   for i in core_interfaces do (showinterface i; format&quot;\\n&quot;)\n\
</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Interfaces.html\">Interfaces</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Other-Interfaces/Other-Interfaces-1.html\">Other Interfaces</a></li>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Language-Reference/Creating-Functions/By-Reference-Parameter-Passing.html\">By Reference Parameter Passing</a></li>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Language-Reference/Creating-Functions/Dereferencing-Operator.html\">Dereferencing Operator</a></li>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Language-Reference/Creating-Functions/Visible-Class-For-Reference.html\">Visible Class For &#x27;&amp;&#x27; Reference Values</a></li>\n\
          <li><a class=\"link\" href=\"../../MAXScript-Tools-and-Interaction/Interacting-with-the-3ds-Max/Action-Manager-and-Shortcut/Action-Manager.html\">Action Manager</a></li>\n\
          <li><a class=\"link\" href=\"../Identifying-and-Accessing/Class-and-Object-Inspector.html\">Class and Object Inspector Functions</a></li>\n\
          <li><a class=\"link\" href=\"showInterfaces-Inspector.html\">showInterfaces Inspector Function</a></li>\n\
          <li><a class=\"link\" href=\"GetInterfaces-Interface.html\">GetInterfaces Interface Inspector</a></li>\n\
          <li><a class=\"link\" href=\"showInterface-Inspector-Function.html\">showInterface Inspector Function</a></li>\n\
          <li><a class=\"link\" href=\"Increasing-Performance-when.html\">Increasing Performance when Searching for Interfaces and Methods</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";