var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-5ECE930B-32E0-42A7-9344-B9E0A2F73171\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Never get a single pixel when you can get a whole line</h1></div><p>The <a href=\"./../../../MAXScript-Language-Reference/Values/Bitmap-Values/Bitmap-Values-1.html\">getPixels and setPixels functions</a> for reading and writing pixels from / to bitmaps are rather slow.</p>\n\
<p>When changing a large number or even all pixels of a bitmap, it is a good idea to perform the <code>getPixels()</code> and <code>setPixels()</code> just once for each horizontal line and do the rest of the work with the resulting array elements.</p>\n\
<p>In the following examples, a 1000x1000 pixels bitmap should be altered by changing every single pixel. This means reading and writing back one million pixels.</p>\n\
<p>In the first example, every single pixel will be read separately, multiplied by a number and written back. We will stop the time in order to compare to the optimized version.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE 1 - UNOPTIMIZED SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   b = bitmap 1000 1000 color:red--create new bitmap with red background\n\
   st = timestamp()--get start time in milliseconds\n\
   for y = 1 to 1000 do--go through all lines in the bitmap\n\
   (\n\
     for x = 1 to 1000 do--go through all pixels in a line\n\
     (\n\
       pixels = getPixels b [x-1,y-1] 1--read one pixel from X,Y\n\
       pixels[1] *= (x+y)/2000.0--alter the pixel\n\
       setPixels b [x-1,y-1] pixels--write modified pixel back\n\
     )--end x loop\n\
   )--end y loop\n\
   et = timestamp()--get end time in milliseconds\n\
   print (et-st)--print time to finish\n\
   display b--show a nice black-red gradient\n\
</code></pre><p>   Running this script on a 2.1GHz system resulted in a time stamp of 16094 milliseconds.</p>\n\
</blockquote>\n\
<p>In the second example, instead of reading every single pixel, will be read every line and process all its pixels inside the array returned by <code>getPixels()</code> before writing the whole line back to the bitmap. This means we will make only 1000 read and 1000 write calls instead of one million reads and one million writes.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE 2 - OPTIMIZED SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   b = bitmap 1000 1000 color:red--create new bitmap with red background\n\
   st = timestamp()--get start time in milliseconds\n\
   for y = 1 to 1000 do--go through all lines in the bitmap\n\
   (\n\
     pixels = getPixels b [0,y-1] 1000--read all 1000 pixels of a single line\n\
     for x = 1 to 1000 do--go through all pixels in the line\n\
     (\n\
       pixels[x] *= (x+y)/2000.0--alter the pixel\n\
     ) \n\
     setPixels b [0,y-1] pixels--write back the complete modified line\n\
   )\n\
   et = timestamp()--get end time in milliseconds\n\
   print (et-st)--print time to finish\n\
   display b--show the same nice black-red gradient\n\
</code></pre><p>   Running the optimized script on the same 2.1GHz system resulted in a time stamp of 4937 milliseconds - more than three times faster!</p>\n\
</blockquote>\n\
<p>Since <strong>3ds Max 2010</strong>, <a href=\"./../../../MAXScript-Language-Reference/Values/Bitmap-Values/Bitmap-Values-1.html\">the pasteBitmap() function</a> can be used in a custom <code>function:</code> mode to perform per-pixel operations using a MAXScript function. This method does not involve the <code>getPixels()</code> and <code>setPixels()</code> calls but does the same and is also great for bothcompositing two bitmaps together and for manipulating pixels ina single bitmap as in our test example.We simply pass the same bitmap twice. The result is normally stored in the second bitmap, in this case it is identical to the first bitmap so we effectively read and write using one bitmap value:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE 3 - ALTERNATIVE SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn compfn c1 p1 c2 p2 =\n\
   (\n\
     c1 * ((p1.x+p1.y)/2000.0)\n\
   )\n\
   b = bitmap 1000 1000 color:red--create new bitmap with red background\n\
   st = timestamp()--get start time in milliseconds\n\
   pasteBitmap b b [0,0] [0,0] type:#function function:compfn--call the function per pixel\n\
   et = timestamp()--get end time in milliseconds\n\
   print (et-st)--print time to finish\n\
   display b--show the same nice black-red gradient\n\
</code></pre><p>   Running the alternative script on the same 2.1GHz system resulted in a time stamp of 3828 milliseconds - a second faster than the optimized script above!</p>\n\
</blockquote>\n\
<p>The following table shows the times in milliseconds for the three examples at different image resolutions:</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th></th>\n\
<th>2000x2000</th>\n\
<th>1000x1000</th>\n\
<th>500x500</th>\n\
<th>250x250</th>\n\
<th>125x125</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><strong>EXAMPLE 1</strong></td>\n\
<td>64657</td>\n\
<td>16094</td>\n\
<td>3578</td>\n\
<td>906</td>\n\
<td>203</td>\n\
</tr>\n\
<tr>\n\
<td><strong>EXAMPLE 2</strong></td>\n\
<td>20000</td>\n\
<td>4937</td>\n\
<td>1266</td>\n\
<td>344</td>\n\
<td>62</td>\n\
</tr>\n\
<tr>\n\
<td><strong>EXAMPLE 3</strong></td>\n\
<td>15187</td>\n\
<td>3828</td>\n\
<td>906</td>\n\
<td>172</td>\n\
<td>47</td>\n\
</tr>\n\
</tbody></table>\n\
<p><strong>Previous Tip</strong></p>\n\
<p><a href=\"Use-the-flagForeground-node.html\">Use the &apos;flagForeground&apos; node viewport state method</a></p>\n\
<p><strong>Next Tip</strong></p>\n\
<p><a href=\"Only-calculate-once-if-possible.html\">Only calculate once if possible</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Make-It-Faster-.html\">How To Make It Faster?</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";