var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-2420F55C-E839-4F6A-9E89-6866D118A4A9\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Optimizing Particle Flow Script Operator For Loops For Speed</h1></div><h2 id=\"a-user-posted-this-code-used-inside-an-on-proceed-handler-\">A user posted this code used inside an On Proceed() handler:</h2><blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   for i in 1 to count do\n\
   (\n\
   pCont.particleIndex = i\n\
   colmult = 80\n\
   pspeed = pCont.particlespeed\n\
   pvel = sqrt (pspeed.x*pspeed.x+pspeed.y*pspeed.y+pspeed.z*pspeed.z)\n\
   pCont.particleAge = pvel*colmult\n\
   )\n\
</code></pre></blockquote>\n\
<h2 id=\"performance-comments-\">Performance Comments:</h2><p>The line,</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">pvel = sqrt (pspeed.x*pspeed.x+pspeed.y*pspeed.y+pspeed.z*pspeed.z)\n\
</code></pre><p>is equivalent to,</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">pvel = length pspeed\n\
</code></pre><p>but is 4 times slower. When calculating inside a Particle Flow operator, you want the fastest method because the code is called thousands and even millions of times, once for each particle.</p>\n\
<p>The first code block evaluates in 3360 ms when run on a particular machine.</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   st = timestamp()\n\
   pspeed = [10,2,3]\n\
   for i = 1 to 1000000 do\n\
   sqrt (pspeed.x*pspeed.x+pspeed.y*pspeed.y+pspeed.z*pspeed.z)\n\
   format &quot;%\\n&quot; (timestamp()-st)\n\
   )\n\
</code></pre></blockquote>\n\
<p>However, the same code using the length() method,</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   st = timestamp()\n\
   pspeed = [10,2,3]\n\
   for i = 1 to 1000000 do\n\
   length pspeed\n\
   format &quot;%\\n&quot; (timestamp()-st)\n\
   )\n\
</code></pre></blockquote>\n\
<p>evaluates in just 813 ms on the same machine, which is 4.13 times faster.</p>\n\
<p>In addition, assigning to a temporary variable pvel adds 200 ms to 1 million calls in both cases, so rewriting the original code block to</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   for i = 1 to count do(\n\
   pCont.particleIndex = i\n\
   pCont.particleAge = length (pCont.particlespeed) * 80\n\
   )\n\
   )\n\
</code></pre></blockquote>\n\
<p>without any intermediate variables can shave off up to 600 ms per one million calls on top of the speed up from using the length() method.</p>\n\
<p>Here is another test to simulate this. It contains three variable assignments within the loop and the original velocity calculation:</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   st = timestamp()\n\
   for i = 1 to 1000000 do (\n\
   pspeed = [10,2,3]\n\
   colmult = 80\n\
   pvel = sqrt (pspeed.x*pspeed.x+pspeed.y*pspeed.y+pspeed.z*pspeed.z)\n\
   pvel * colmult\n\
   )\n\
   format &quot;%\\n&quot; (timestamp()-st)\n\
   )\n\
</code></pre></blockquote>\n\
<p>The above code is very similar to what the original code was doing (minus actually reading and writing particle values). It evaluates in 5750 ms.</p>\n\
<p>In contrast,</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   st = timestamp()\n\
   for i = 1 to 1000000 do (\n\
   pspeed = [10,2,3]\n\
   colmult = 80\n\
   pvel = length pspeed\n\
   pvel * colmult\n\
   )\n\
   format &quot;%\\n&quot; (timestamp()-st)\n\
   )\n\
</code></pre></blockquote>\n\
<p>executes in 3200 ms just because we got rid of the three additions and three multiplications within the MAXScript code. Length() does the same internally, but is implemented in C++ and is thus much faster.</p>\n\
<p>The inline version of the test code without any intermediate variables that calculates the same value at the end needs only 1400 ms.</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   st = timestamp()\n\
   for i = 1 to 1000000 do (\n\
   length [10,2,3] * 80\n\
   )\n\
   format &quot;%\\n&quot; (timestamp()-st)\n\
   )\n\
</code></pre></blockquote>\n\
<p>In short, avoid using local intermediate variables within your PFlow scripts and use built-in methods to get the best performance as in:</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   for i = 1 to pCont,Numparticles() do (\n\
   pCont.particleIndex = i\n\
   pCont.particleAge = length (pCont.particlespeed) * 80\n\
   )\n\
   )\n\
</code></pre></blockquote>\n\
<p><strong>Previous Tip</strong></p>\n\
<p><a href=\"For-Loop-Collect-Vs-For-Loop-Do.html\">For Loop Collect Vs For Loop Do Performance</a></p>\n\
<p><strong>Next Tip</strong></p>\n\
<p><a href=\"Using-Bsearch-For-Fast-Table.html\">Using Bsearch For Fast Table Lookup</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Make-It-Faster-.html\">How To Make It Faster?</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";