var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-BFE0D012-E8F6-4903-B3C5-ABDA06F4F18C\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>How To ... Create Custom Mesh Object</h1></div><p>MAXScript lets you create editable mesh objects by supplying lists of vertices and faces to the mesh constructor.</p>\n\
<p>This topic shows how to write a simple mesh-generation script to create a circular 2D mesh object, similar to a sun symbol.</p>\n\
<p><strong>Related Topics:</strong></p>\n\
<p><a href=\"./../MAXScript-Language-Reference/Collections/Collection-Types/Array-Values.html\">Working with Arrays</a></p>\n\
<p><a href=\"./../3ds-Max-Objects-and-Interfaces/Editable-Meshes-Splines-Patches/Editable_Mesh-and-TriMesh/Editable_Mesh-GeometryClass-and.html\">Creating Editable Mesh objects</a></p>\n\
<blockquote>\n\
<p>   <strong>NATURAL LANGUAGE</strong></p>\n\
<p>   Define the basic parameters of the mesh object</p>\n\
<p>   Generate all vertices of the mesh and store in an array</p>\n\
<p>   Generate all faces by indexing the vertices and store in another array</p>\n\
<p>   Call the mesh constructor with the two arrays as parameters.</p>\n\
</blockquote>\n\
<blockquote>\n\
<p>   <strong>MAXSCRIPT</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   radius1 = 100\n\
   radius2 = 10\n\
   width = 10\n\
\n\
   vert_array = #()\n\
   face_array = #()\n\
\n\
   vert_count = 0\n\
   num_faces = 10\n\
\n\
   for a = 0 to (360-(360/num_faces)) by 360/num_faces do\n\
   (\n\
   v1 = [radius1*cos(a+width),radius1*sin(a+width),0]\n\
   v2 = [radius1*cos(a-width),radius1*sin(a-width),0]\n\
   v3 = [radius2*cos(a),radius2*sin(a),0]\n\
   append vert_array v1\n\
   append vert_array v2\n\
   append vert_array v3\n\
\n\
   append face_array [vert_count+1,vert_count+3,vert_count+2]\n\
   vert_count += 3\n\
   )\n\
\n\
   m = mesh vertices:vert_array faces:face_array\n\
</code></pre></blockquote>\n\
<h2 id=\"step-by-step-\">Step-By-Step:</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">radius1 = 100\n\
radius2 = 10\n\
width = 10</code></pre><p>First, we need some variables to define the geometry. The circular object has an inner and outer radius and a width value defining the outer size of the ray.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">vert_array = #()\n\
face_array = #()</code></pre><p>The two arrays store the vertex coordinates and the face definitions.</p>\n\
<p><a href=\"./../MAXScript-Language-Reference/Collections/Collection-Types/Array-Values.html\">Array Values</a></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">vert_count = 0</code></pre><p>The user variable <code>vert_count</code> is set to 0 and will be incremented while creating the faces of the mesh.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">num_faces = 10</code></pre><p>The number of steps is related to the number of faces to create. It is used as step in the following loop.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\"> for a = 0 to (360-(360/num_faces)) by 360/num_faces do\n\
(</code></pre><p>The <code>for a</code> loop counts from 0 to 360 (degrees), minus one step to prevent overlap, with a user-defined step provided by the <code>num_faces</code> variable. This variable defines the number of &quot;rays&quot; in the mesh. For example, to get 10 rays we need 360/10 = 36 degrees per ray. The <code>a</code> value will be incremented nine times by the value of 36 to make a full turn in degrees.</p>\n\
<p><a href=\"./../MAXScript-Language-Reference/Controlling-Program-Flow/For-Loop.html\">For Loop</a></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">v1 = [radius1*cos(a+width),radius1*sin(a+width),0]\n\
v2 = [radius1*cos(a-width),radius1*sin(a-width),0]\n\
v3 = [radius2*cos(a),radius2*sin(a),0]</code></pre><p>Now we will define the 3 vertices for a single face.</p>\n\
<p>The Vertices v1 and v2 are at the outer side of the object.</p>\n\
<p>The Vertex v3 is closer to the center (at distance equal to radius2).</p>\n\
<p>When radius3 is 0, all v3 vertices of all faces are coincident at the center.</p>\n\
<p>We use the equation of the circle which is:</p>\n\
<p>X = CenterX + Radius * Cos(A); Y = CenterY + Radius * Sin(A); Z = 0.0</p>\n\
<p>where A is an angle changing from 0 to 2*Pi. In our case, CenterX and CenterY are 0.0.</p>\n\
<p>By adding and subtracting the width value to/from the angle <code>a</code> , we get the outer two vertices offset from the actual angle used by the third vertex. The third coordinate Z is always 0 in our case &#x2013; the mesh will reside on the ground XY plane.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">append vert_array v1\n\
append vert_array v2\n\
append vert_array v3</code></pre><p>We add all 3 new vertices to the vertex array.</p>\n\
<p><a href=\"./../MAXScript-Language-Reference/Collections/Collection-Types/Array-Values.html\">Array Values</a></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">append face_array [vert_count+1,vert_count+3,vert_count+2]</code></pre><p>Now we can store a face in the face array, listing the 3 vertex indices as a Point3 value in Counter-Clockwise order. This will create the face with a normal pointing up. Since <code>vert_count</code> is still 3 less than the number of vertices in the <code>vert_array</code> , we can add 1, 2 and 3 to its current value to get the correct indices needed for the face definition.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">vert_count += 3</code></pre><p>Now we increase the vertex counter by 3. Next face definition will use the correct indices again. The same could be expressed as <code>vert_count = vert_count + 3</code></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)\n\
m = mesh vertices:vert_array faces:face_array</code></pre><p>Now we can create a new Editable mesh by calling the <code>mesh</code> constructor and supplying the vertex and face arrays.</p>\n\
<p><a href=\"./../3ds-Max-Objects-and-Interfaces/Editable-Meshes-Splines-Patches/Editable_Mesh-and-TriMesh/Editable_Mesh-GeometryClass-and.html\">Editable_Mesh : GeometryClass and TriMesh : Value</a></p>\n\
<h2 id=\"using-the-script\">Using the Script</h2><p>If you evaluate the script now, a new mesh object will appear in the scene. Changing the initial radius and width values lets you create multiple variations of the mesh.</p>\n\
<h2 id=\"where-to-go-from-here\">Where to go from here</h2><p>This was just a sketch of a mesh-creating script. In a second step, we will enhance this code to generate a real geometry plug-in script operating interactively from within the Create tab.</p>\n\
<p><a href=\"How-To-Create-Scripted-Geometry.html\">How To ... Create Scripted Geometry Plug-in</a></p>\n\
<p>Possible things to try yourself would be assigning different material IDs to the separate faces, assigning UVW coordinates to the mesh etc.</p>\n\
<p><strong>Back to</strong></p>\n\
<p><a href=\"How-To-Introduction-to-the.html\">&quot;How To&quot; Tutorials Index Page</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Practical-Examples.html\">How To - Practical Examples</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";