var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-DEA74F18-7B83-4459-ABFB-F36A3BD49BD7\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>How To ... Copy The Last Rendered Image To The Windows Clipboard</h1></div><p>The following tutorial demonstrates the use of the <strong>System.Windows.Forms.Clipboard</strong> DotNetClass to read and write bitmaps from and to the Windows Clipboard.</p>\n\
<p>The script can be saved into one of the Startup folders to be evaluated on startup. The custom function will be executed by a PostRender callback each time the renderer finishes, putting the last rendered image into the Windows Clipboard automatically.</p>\n\
<p><strong>Related topics:</strong></p>\n\
<p><a href=\"./../../Interaction-with-Other/DotNet-In-MAXScript.html\">DotNet In MAXScript</a></p>\n\
<p><a href=\"./../../Interaction-with-Other/DotNet-In-MAXScript/dotnetclass.html\">dotNetClass</a></p>\n\
<p><a href=\"./../../Interaction-with-Other/DotNet-In-MAXScript/Selected-DotNet-Classes-and/dotNetClass-System-Windows-Forms.html\">dotNetClass:System.Windows.Forms.Clipboard</a></p>\n\
<blockquote>\n\
<p>   <strong>NATURAL LANGUAGE</strong></p>\n\
<p>   Define a global function to be called by a general callback after each rendering.</p>\n\
<p>   Get the DotNet class exposing Windows Clipboard functionality</p>\n\
<p>   Get the last rendered image via MAXScript</p>\n\
<p>   Save the image to disk</p>\n\
<p>   Convert the image to a DotNet Image</p>\n\
<p>   Put the image into the Windows Clipboard</p>\n\
<p>   Release any memory related to the image and delete the file from disk</p>\n\
<p>   Returns true if the clipboard now contains an image</p>\n\
<p>   Unregister any callbacks with the same ID</p>\n\
<p>   Register a PostRender callback with a custom ID that calls the function</p>\n\
</blockquote>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn CopyRenderToClipboard =\n\
   (\n\
     clipboardClass = dotNetClass &quot;System.Windows.Forms.Clipboard&quot;\n\
     b = getLastRenderedImage()\n\
     theFileName = GetDir #image +&quot;\\\\_renderToClipboard.bmp&quot;\n\
     b.filename = theFileName\n\
     save b\n\
     close b\n\
     theImage = dotNetClass &quot;System.Drawing.Image&quot;\n\
     theBitmap = theImage.FromFile theFileName\n\
     clipboardClass.setImage theBitmap\n\
     theBitmap.Dispose()\n\
     deleteFile theFileName\n\
     clipboardClass.ContainsImage()\n\
   )--end fn\n\
\n\
   callbacks.removeScripts id:#CopyRenderToClipboard\n\
   callbacks.addScript #PostRender &quot;CopyRenderToClipboard()&quot; id:#CopyRenderToClipboard\n\
</code></pre></blockquote>\n\
<h2 id=\"step-by-step\">Step-By-Step</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">fn CopyRenderToClipboard = (</code></pre><p>Define a global function. We will place the script in a startup folder, so we want the function to be visible in the global scope.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">clipboardClass = dotNetClass &quot;System.Windows.Forms.Clipboard&quot;</code></pre><p>Get the Clipboard dotNetClass from System.Windows.Forms</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">b = getLastRenderedImage()</code></pre><p>Get the last rendered image</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">theFileName = GetDir #image + &quot;\\\\_renderToClipboard.bmp&quot;</code></pre><p>Define a temporary file name.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">b.filename = theFileName</code></pre><p>Set the bitmap&apos;s file name to the one defined in the previous step.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">save b</code></pre><p>Save the bitmap to disk under the assigned file name.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">close b</code></pre><p>Close the bitmap.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">theImage = dotNetClass &quot;System.Drawing.Image&quot;</code></pre><p>Get the Image dotNetClass from System.Drawing</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">theBitmap = theImage.FromFile theFileName</code></pre><p>Get the saved image from file as bitmap</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">clipboardClass.setImage theBitmap</code></pre><p>Copy the image to the clipboard by calling the <code>setImage</code> method exposed by the clipboard class.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">theBitmap.Dispose()</code></pre><p>Release the bitmap - it is already in the Windows Clipboard!</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">deleteFile theFileName</code></pre><p>Delete the temporary file from disk.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">clipboardClass.ContainsImage()</code></pre><p>Return true if the Windows Clipboard contains an image, which we can assume is our image.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)--end fn</code></pre><p>This is the end of the global function.</p>\n\
<p>Now we will register a general callback to call the function whenever the renderer finishes.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.removeScripts id:#CopyRenderToClipboard</code></pre><p>First we make sure we remove any callbacks with the same ID as the one we are about to register. This way, you can run the script multiple times (for example while further developing the function or making changes to the code in general).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbacks.addScript #PostRender &quot;CopyRenderToClipboard()&quot; id:#CopyRenderToClipboard</code></pre><p>Then we register a PostRender callback to run the function whenever the renderer finishes.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Develop-Tools-Using.html\">How To ... Develop Tools Using DotNet</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../How-To-Introduction-to-the.html\">How To - Introduction to the Tutorials</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";