var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-853014AC-9DC8-4F5C-B14B-3643E5925AD8\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>How To ... Develop a Vertex Renderer</h1></div><p>The tutorial will demonstrate the basics of rendering - transforming objects from 3D space into 2D images.To make it simple and fast (realtime!), the script will draw only vertices. The image will be redrawn every time the viewport is updated - changing the view, panning, zooming and orbiting will be reflected by the vertex renderer instantly!</p>\n\
<p><strong>Related topics:</strong></p>\n\
<p><a href=\"./../MAXScript-Tools-and-Interaction/Interacting-with-the-3ds-Max/Viewports/Accessing-Active-Viewport-Info.html\">Accessing Active Viewport Info, Type, and Transforms</a></p>\n\
<blockquote>\n\
<p>   <strong>NATURAL LANGUAGE</strong></p>\n\
<p>   Create a macroScript.</p>\n\
<p>   Define a flag variable to remember whether the renderer is enabled or disabled</p>\n\
<p>   Define local variables to hold the render size and the front and back buffer images</p>\n\
<p>   Define a function to be called when the viewport redraws.</p>\n\
<p>   In the function,</p>\n\
<ul>\n\
<li><p>Copy the back buffer into the front buffer</p>\n\
</li>\n\
<li><p>Go through all geometry objects and get their mesh</p>\n\
</li>\n\
<li><p>Get every vertex of the mesh, transform into image coordinates using the current view transformation matrix</p>\n\
</li>\n\
<li><p>Draw a pixel for the vertex in the front buffer</p>\n\
</li>\n\
<li><p>When done, display the image</p>\n\
</li>\n\
</ul>\n\
<p>   Return the flag from the on isChecked handler.</p>\n\
<p>   When executed,</p>\n\
<ul>\n\
<li><p>Invert the flag</p>\n\
</li>\n\
<li><p>Redefine the render size variables and the bitmaps</p>\n\
</li>\n\
<li><p>Register the viewport redraw callback function if the flag is true, unregister if false.</p>\n\
</li>\n\
</ul>\n\
</blockquote>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   macroScript VertexRender category:&quot;HowTo&quot;\n\
   (\n\
   local VertexRendererEnabled = false\n\
   local screen_width, screen_height, back_vfb, front_vfb\n\
   fn VertexRendererFunction =\n\
   (\n\
   st = timestamp()\n\
   copy back_vfb front_vfb\n\
   for o in geometry where classof o != TargetObject do \n\
   (\n\
   theMesh = snapshotAsMesh o\n\
   dot_color = #(o.wirecolor)\n\
   theVertCount = theMesh.numverts\n\
   for v = 1 to theVertCount do\n\
   ( \n\
   thePos = (getVert theMesh v)* viewport.getTM()\n\
   screen_origin = mapScreenToView [0,0] (thePos.z) [screen_width,screen_height]\n\
   end_screen = mapScreenToView [screen_width,screen_height] (thePos.z) [screen_width,screen_height]\n\
   world_size = screen_origin-end_screen\n\
   x_aspect = screen_width/(abs world_size.x)\n\
   y_aspect = screen_height/(abs world_size.y)\n\
   screen_coords = point2 (x_aspect*(thePos.x-screen_origin.x)) (-(y_aspect*(thePos.y-screen_origin.y)))\n\
   setPixels front_vfb screen_coords dot_color\n\
   )--end v loop\n\
   )--end o loop\n\
   display front_vfb \n\
   et = timestamp()\n\
   pushPrompt (&quot;VertRender &quot;+(1000.0/(et-st)) as string +&quot; fps&quot;)\n\
   )--end fn\n\
\n\
   on isChecked return VertexRendererEnabled\n\
\n\
   on execute do\n\
   (\n\
   VertexRendererEnabled =not VertexRendererEnabled\n\
\n\
   if VertexRendererEnabled then\n\
   (\n\
   screen_width=RenderWidth\n\
   screen_height=RenderHeight\n\
   back_vfb = bitmap screen_width screen_height\n\
   front_vfb = bitmap screen_width screen_height\n\
   registerRedrawViewsCallback VertexRendererFunction\n\
   RedrawViews() \n\
   ) \n\
   else\n\
   (\n\
   unRegisterRedrawViewsCallback VertexRendererFunction \n\
   close front_vfb \n\
   )\n\
   )--end on execute\n\
   )--end script\n\
</code></pre></blockquote>\n\
<p><img src=\"./images/GUID-B22E1D14-6E5C-4D81-ADD9-E528B2F2E784-low.gif\" alt></p>\n\
<h2 id=\"step-by-step\">Step-By-Step</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">macroScript VertexRender category:&quot;HowTo&quot;\n\
(</code></pre><p>We start by defining a MacroScript with the name VertexRender which will appear in the category &quot;HowTo&quot;. This will be an advanced macroScripts which will have <code>on execute do</code> or <code>on is Checked do</code> event handlers.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">localVertexRendererEnabled= false</code></pre><p>We define a local variable to hold a flag determining whether the renderer is active or not. In the beginning, the variable will be set to false.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">local screen_width,screen_height, back_vfb,front_vfb</code></pre><p>We define a couple more local variables to hold the width and height of the image and the two bitmaps - the back buffer and the front buffer (see below for more info).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">fn VertexRendererFunction =\n\
(</code></pre><p>This is the function that will be executed when the renderer is enabled and the viewport is being updated.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">st = timestamp()</code></pre><p>To be able to output info on the rendering speed, we will use the timestamp function to get the current system time.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">copy back_vfb front_vfb</code></pre><p>Then we will copy the back buffer into the front buffer, basically clearing the previous image. The back buffer could be changed to hold a background image, and the function would draw on top of it... For now, the back buffer image is set to black.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for o in geometry where classof o != TargetObject do\n\
(</code></pre><p>We loop through all scene geometry objects that are not target objects of lights, cameras etc. Note that we are not checking for hidden objects - you could add this to the code as an exercise! Right now, all objects will be drawn, even the hidden ones!</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">theMesh = snapshotAsMesh o</code></pre><p>Then we snapshot the objects as mesh into memory. This includes all modifiers AND space warps applied to the object.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">dot_color = #(o.wirecolor)</code></pre><p>We define an array to be used for painting the vertices and use the wireframe color to draw.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">theVertCount = theMesh.numverts</code></pre><p>To speed things up, we read the number of vertices in the mesh once. We will use this value below to limit the vertex loop:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for v = 1 to theVertCount do\n\
( </code></pre><p>The v variable will count from 1 to the number of vertices.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">thePos = (getVert theMesh v)* viewport.getTM()</code></pre><p>The position of the v-th vertex has to be multiplied by the transformation matrix of the viewport. (If the viewport is a Camera, the viewport.getTM() returns the inverse of the camera&apos;s transformation matrix. If the viewport is a Perspective or Ortho view, the matrix is already inverted). By multiplying the vertex position by this matrix, we transform the 3D space coordinates into camera space!</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">screen_origin = mapScreenToView [0,0] (thePos.z) [screen_width,screen_height]</code></pre><p>Now we need to know the size of the projection plane with pixel size equal to the rendering image when placed at the depth of the current vertex. To calculate this, we will need the two extremes (upper left and lower right) points of the plane as 3D points in camera space.</p>\n\
<p>Since thePos is already in camera coordinates, the.Z coordinate is the Z-depth of the point!The return value of <code>mapScreenToView</code> is a 3D point in camera space corresponding to the upper left corner of the projection plane.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">end_screen = mapScreenToView [screen_width,screen_height] (thePos.z) [screen_width,screen_height]</code></pre><p>We do the same for the lower right corner of the projection plane.</p>\n\
<p>Here is what we just did and why:</p>\n\
<p>As you know, the camera has a cone (frustrum) with the top at the position of the camera. If the projection plane would &quot;travel&quot; along the Z axis of the camera, the farther it gets from the &quot;eye&quot;, the larger it becomes in world units, but the number of pixels in the final image remains the same.</p>\n\
<p><img src=\"./images/GUID-F18B8243-7BF3-4894-8D32-45DE2884F9CA-low.gif\" alt></p>\n\
<p>On the screenshot, you can see a camera and two projection planes (shown as red X). You can also see that the size of the projection plane is increasing as it gets farther from the camera.</p>\n\
<p>The farther projection plane is exactly at the depth of the teapot&apos;s position in camera space.</p>\n\
<p>The red sphere in the upper left corner of the projection plane corresponds to the <code>screen_origin</code> coordinate calculated above. To create the image, the <code>screen_origin</code> value was calculated, then the MAXScript line of code</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">sphere pos:(screen_origin*$Camera01.transform)</code></pre><p>was called to create a sphere in world coordinates representing that point. ( <code>screen_origin</code> was originally in camera space, multiplying it by the camera&apos;s transformation matrix transforms it back into world coordinates!)</p>\n\
<p>Same with the blue sphere - it represents the <code>end_screen</code> coordinate.</p>\n\
<p>Here is what the camera sees when rendering the view - notice the red and blue spheres appearing exactly where expected:</p>\n\
<p><img src=\"./images/GUID-2553A757-0412-469C-973C-FB5806851F12-low.gif\" alt></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">world_size = screen_origin-end_screen</code></pre><p>Now we know the two extremes of the projection plane and can calculate the width and height of the projection plane in world units. We also know the size of the plane in pixels, so we can determine the relationship between pixels and world units!</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">x_aspect = screen_width/(abs world_size.x)\n\
y_aspect = screen_height/(abs world_size.y)</code></pre><p>Using the size of the plane, we can calculate the aspect values - basically the number of pixels corresponding to one world unit along X and Y.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">screen_coords= point2 (x_aspect*(thePos.x-screen_origin.x)) (-(y_aspect*(thePos.y-screen_origin.y)))</code></pre><p>Using the aspect values, we can convert the X and Y values of the vertex position into actual pixel positions. Note that the Y coordinate is inverted, and the X position has to be offset by the screen_origin because the camera axis (the center of the image) is located at coordinates [0,0], while the actual origin of the image in MAXScript is in the upper left corner...</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">setPixels front_vfbscreen_coordsdot_color</code></pre><p>Now we can draw a pixel in the front buffer at the location of the current vertex using the color array defined above using the wireframe color.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)--end v loop\n\
)--end o loop\n\
display front_vfb </code></pre><p>When all vertices of all objects have been processed, we can update the display.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">et = timestamp()</code></pre><p>Then we can stop the time again to see how long it took to draw once.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">pushPrompt (&quot;VertRender &quot; +(1000.0/(et-st)) as string + &quot; fps&quot;)</code></pre><p>The result will be output to the status bar.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)--end fn\n\
on isChecked returnVertexRendererEnabled</code></pre><p>When the macroScript is placed on a toolbar, menu or QuadMenu, its checked status will be determined by the boolean value stored in the local variable we defined in the beginning...</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on execute do (</code></pre><p>When the user presses the button / selects the menu item...</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">VertexRendererEnabled = not VertexRendererEnabled</code></pre><p>...we flip the state of the variable - if it was true it becomes false and vice-versa.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">if VertexRendererEnabled then\n\
(</code></pre><p>If the script was just enabled, then</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">screen_width=RenderWidth\n\
screen_height=RenderHeight</code></pre><p>we read the current size values of the Render Scene dialog. You could replace these values with your fixed values, for example 320 and 240 to have a fixed size regardless of the renderer settings...</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">back_vfb = bitmap screen_width screen_height\n\
front_vfb = bitmap screen_width screen_height</code></pre><p>Then we use these size values to define two bitmaps. The first is the back buffer used to clear the image before drawing. Once again, you could use an openBitmap call here to load a background image from disk to draw on top of it!</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">registerRedrawViewsCallback VertexRendererFunction</code></pre><p>Here we register the function we defined above as a viewport redraw callback. Each time the <strong>3ds Max</strong> viewport is redrawn, the function will be called and update the rendered image!</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">VertexRendererFunction() </code></pre><p>To initialize the display, we call the function once.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">) \n\
else\n\
(\n\
unRegisterRedrawViewsCallback VertexRendererFunction </code></pre><p>If the renderer was already open, it has to close now. We unregister the callback function to avoid further updates,</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">close front_vfb </code></pre><p>and close the image to undisplay it.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)\n\
)--end on execute\n\
)--end script</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Practical-Examples.html\">How To - Practical Examples</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"How-To-Introduction-to-the.html\">How To - Introduction to the Tutorials</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";