var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-99E3B658-D86C-4FD6-9B37-753AEF05FB74\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>DotNet Objects and Classes Life Time Control</h1></div><p>In versions prior to <strong>3ds Max 2010</strong>, performing MAXScript garbage collection can delete the DotNet event handlers too early.</p>\n\
<p>With this method available in <strong>3ds Max 2010 and higher</strong>, this can be avoided by setting the life time control of the corresponding objects to be handled by DotNet instead of MAXScript.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">dotNet.setLifetimeControl {&lt;dotNetObject&gt; | &lt;dotNetClass&gt;} {#mxs | #dotnet}</code></pre><p>Controls whether the MAXScript value wrapping the DotNet object or DotNetclass can be garbage collected when there are no MAXScript references to the value ( <code>#mxs</code> ), or whether the DotNet object is destroyed before the MAXScript value can be garbage collected ( <code>#dotnet</code> ).</p>\n\
<p>By default, a dotNetObject value holds a <strong>strong reference</strong> to the DotNet object, and the DotNet object is deleted only when the dotNetObject value is deleted. This is equivalent to specifying <code>#mxs</code> as second argument in the above method.</p>\n\
<p>If you specify <code>#dotnet</code> as the second argument, the dotNetObject cannot be collected until the DotNet object is deleted. The dotNetObject value holds a weak reference to the DotNet object and the DotNet object is deleted when no other DotNet objects hold a reference to it and a DotNet garbage collection occurs.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE 1 - HANDLERS DELETED</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   function whenButtonIsPressed =\n\
   (\n\
   print &quot;Pressed from a .NET button&quot;\n\
   )\n\
   function formIsClosed =\n\
   (\n\
   print &quot;form closed&quot;\n\
   )\n\
   local mButton = dotNetObject &quot;System.windows.forms.button&quot;\n\
   mButton.text = &quot;.NET Button&quot;\n\
   mButton.size = dotNetObject &quot;System.Drawing.Size&quot; 100 100\n\
   mButton.location = dotNetObject &quot;System.Drawing.Point&quot; 50 50\n\
   local hForm = dotNetObject &quot;System.Windows.Forms.Form&quot;\n\
   hForm.controls.add mButton\n\
   dotNet.addEventHandler mButton &quot;click&quot; whenButtonIsPressed\n\
   dotNet.addEventHandler hForm &quot;Closed&quot; formIsClosed\n\
   hForm.show()\n\
   ok\n\
   )\n\
   gc()\n\
</code></pre><p>   Evaluating the above code creates a DotNet Form with a DotNet button and two event handlers - one to print a message to the Listener when the button is pressed and one to print a message when the Form is closed.</p>\n\
<p>   Because of the MAXScript Garbage Collection performed after the creation of the Form, both handlers are deleted immediately and nothing is printed when pressing the button or closing the form.</p>\n\
</blockquote>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE2 - HANDLERS PRESERVED</strong></p>\n\
<p>   By switching the life time control of the button and the form to DotNet, it is ensured that the event handlers of both buttons are preserved after a MAXScript Garbage Collection:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   function whenButtonIsPressed =\n\
   (\n\
   print &quot;Pressed from a .NET button&quot;\n\
   )\n\
   function formIsClosed =\n\
   (\n\
   print &quot;form closed&quot;\n\
   )\n\
   local mButton = dotNetObject &quot;System.windows.forms.button&quot;\n\
   mButton.text = &quot;.NET Button&quot;\n\
   mButton.size = dotNetObject &quot;System.Drawing.Size&quot; 100 100\n\
   mButton.location = dotNetObject &quot;System.Drawing.Point&quot;50 50\n\
   local hForm = dotNetObject &quot;System.Windows.Forms.Form&quot;\n\
   hForm.controls.add mButton\n\
   dotNet.addEventHandler mButton &quot;click&quot; whenButtonIsPressed\n\
   dotNet.addEventHandler hForm &quot;Closed&quot; formIsClosed\n\
   dotNet.setLifetimeControl mButton #dotnet\n\
   dotNet.setLifetimeControl hForm #dotnet\n\
   hForm.show()\n\
   ok\n\
   )\n\
   gc()\n\
</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../DotNet-In-MAXScript.html\">DotNet In MAXScript</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../DotNet-In-MAXScript.html\">DotNet In MAXScript</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";