var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-5A84508D-6F9E-4E18-8C0A-81F91794DA47\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>SafeArrayWrapper Class</h1></div><p>Instances of <strong>SafeArrayWrapper</strong> are creatable by the user and are returned by OLE methods that return SAFEARRAY values. Available in <strong>3ds Max 8 and higher</strong>.</p>\n\
<p><strong>Constructor:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">SafeArrayWrapper ()\n\
SafeArrayWrapper &lt;data array&gt;\n\
SafeArrayWrapper &lt;data array&gt; &lt;lBounds array&gt;</code></pre><p><strong>Properties:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;SafeArrayWrapper&gt;.dataArray</code></pre><p>Data array. Might be &apos;undefined&apos;. The dataArray is a set of nested arrays. The dataArray must be &quot;square&quot; (the number of elements in the dataArray at any given dimension is constant). For example,</p>\n\
<p>dataArray[i].count is constant for i = 1 to dataArray.count, dataArray[i][j].count is constant for j = 1 to dataArray[2].count.</p>\n\
<p>A runtime error will be thrown if dataArray is set to a non-square array. The depth of the array is its dimension. A dimension 2 array is where dataArray[i][j] is a non-array data value. Setting dataArray will change the number of elements in lBoundsArray to match with added elements having the value 0.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;SafeArrayWrapper&gt;.lBoundsArray</code></pre><p>Lower bounds array. Might be &apos;undefined&apos;. Array of size dimensions, where each element contains the lower bound of one dimension of the array. This array is used for creating the OLE SAFEARRAY value from the SafeArrayWrapper. When setting lBoundsArray, its size must be &apos;dimensions&apos; or a runtime error will be thrown.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;SafeArrayWrapper&gt;.dimensions</code></pre><p>Number of dimensions in the data and bounds array. Read only.</p>\n\
<p><strong>Methods</strong>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">copy &lt;SafeArrayWrapper&gt;</code></pre><p>Returns a new SafeArrayWrapper, a deep copy is performed on the data and bounds array.</p>\n\
<p><strong>Notes:</strong></p>\n\
<ul>\n\
<li><p>SafeArrayWrapper values can be passed to OLE methods as arguments and can passed to/from ActiveX control methods. Arrays passed to OLE or ActiveX methods are automatically converted to intermediate SafeArrayWrapper values.</p>\n\
</li>\n\
<li><p>The following additional variant types for values returned by OLE objects can be converted to MAXScriptvalues: VT_UI2, VT_UI4, VT_INT, VT_NULL, and VT_VOID.</p>\n\
</li>\n\
<li><p>Color values are converted to VT_I4 variant values when passed to OLE objects.</p>\n\
</li>\n\
</ul>\n\
<p><strong>Examples:</strong></p>\n\
<p>Example tests which show in part retrieving data from an SQL database using the GetRows method:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">-- open sql database and get rows\n\
DogConn = createOLEObject &quot;ADODB.Connection&quot;\n\
DogConn.Open &quot;driver={SQL Server}; server=dsqedb01.autodesk.com; database=MXSTest&quot;\n\
recordSet = createOLEObject &quot;ADODB.Recordset&quot;\n\
recordSet.Open &quot;SELECT * from TestTable&quot; DogConn\n\
a = recordSet.GetRows()\n\
-- display data properties\n\
a.dataArray\n\
a.lBoundsArray\n\
a.dimensions\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">-- create a new SafeArrayWrapper using data array and lbounds array\n\
b = SafeArrayWrapper #(#(1, 1.2345, &quot;One point two three four five&quot;, undefined, undefined, undefined), #(2, undefined, &quot;Float is null&quot;, undefined, undefined, undefined), #(3, 0.0, &quot; &quot;, undefined, undefined, undefined), #(4, 123.4, undefined, undefined, undefined, undefined)) #(0, 0)\n\
b.dataArray\n\
b.lBoundsArray\n\
b.dimensions\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">-- create a new SafeArrayWrapper using data array\n\
d = SafeArrayWrapper a.dataArray\n\
d.dataArray\n\
d.lBoundsArray\n\
d.dimensions\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">-- create a new SafeArrayWrapper via copy, show it is a deep copy\n\
f = copy a\n\
f.dataArray\n\
f.lBoundsArray\n\
f.dimensions\n\
f.dataArray[1][1] = 999999\n\
f.dataArray\n\
a.dataArray\n\
</code></pre><p>Example of updating an SQL database, where the update method is passed an array or SafeArrayWrapper values:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">DogConn=createOLEObject &quot;ADODB.Connection&quot;\n\
DogConn.Open &quot;driver={SQL Server}; server=dsqedb01.autodesk.com; database=MXSTest&quot;\n\
recordSet = createOLEObject &quot;ADODB.Recordset&quot;\n\
recordSet.Open &quot;SELECT * from TestTable&quot; DogConn 1 3 -- adOpenKeyset adLockOptimistic\n\
fields=recordSet.Fields -- print out column names\n\
count = fields.count\n\
for i = 1 to count do (local item = fields.item (i-1); print item.name)\n\
recordSet.GetRows() -- display current contents\n\
recordSet.AddNew #(&quot;arbitraryFloat&quot;, &quot;arbitraryChar&quot;, &quot;boolean&quot;, &quot;arbitraryInt&quot;) #(pi,&quot;Hello World&quot;,true,42) -- add via array\n\
recordSet.AddNew (SafeArrayWrapper #(&quot;arbitraryFloat&quot;, &quot;arbitraryChar&quot;, &quot;arbitraryInt&quot;)) (SafeArrayWrapper #(e/pi,&quot;Merry xmas!&quot;, 54321)) -- add via SafeArrayWrapper\n\
recordSet.close()\n\
recordSet.Open &quot;SELECT * from TestTable&quot; DogConn\n\
recordSet.GetRows() -- display current contents\n\
</code></pre><p>Example of getting/setting an ActiveX control property with SafeArrayWrapper:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">rollout controlR86 &quot;Microsoft Chart Control 6.0 (SP4) (OLEDB)&quot;\n\
(\n\
activeXControl ax &quot;{3A2B370C-BA0A-11D1-B137-0000F8753F5D}&quot; height:200 width:300 align:#left setupEvents:false\n\
)\n\
createdialog ControlR86 height:250 width:350\n\
data = ControlR86.ax.chartdata -- returns SafeArrayWrapper\n\
ControlR86.ax.chartdata = SafeArrayWrapper #(#(undefined, &quot;R1&quot;, &quot;R2&quot;, &quot;R3&quot;, &quot;R4&quot;, &quot;R5&quot;), #(&quot;C1&quot;, 1, 2, 3, 4, 5), #(&quot;C2&quot;, 1, 2, 3, 4, 5), #(&quot;C3&quot;, 5, 4, 3, 2, 1), #(&quot;C4&quot;, -1, 2, 3, 2, 1))\n\
</code></pre><h3 id=\"conversion-between-maxscript-values-and-ole-variant-values-\">Conversion between MAXScript values and OLE variant values:</h3><p><strong>OLE -&gt; MAXScript:</strong></p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th></th>\n\
<th></th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>VT_EMPTY VT_NULL</code></td>\n\
<td><code>undefined</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>VT_VOID</code></td>\n\
<td><code>OK</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>VT_UI1 VT_UI2 VT_UI4 VT_I2 VT_I4 VT_INT</code></td>\n\
<td><code>Integer</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>VT_R4 VT_R8 VT_CY</code></td>\n\
<td><code>Float</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>VT_BSTR</code></td>\n\
<td><code>String</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>VT_BOOL</code></td>\n\
<td><code>Boolean</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>VT_DISPATCH</code></td>\n\
<td><code>OLEObject</code></td>\n\
</tr>\n\
<tr>\n\
<td>`VT_ARRAY</td>\n\
<td>VT_VARIANT`</td>\n\
</tr>\n\
</tbody></table>\n\
<p><strong>MAXScript -&gt; OLE:</strong></p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th></th>\n\
<th></th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><code>SafeArrayWrapper</code> <code>Array</code></td>\n\
<td><code>VT_ARRAY</code> | <code>VT_VARIANT</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>Integer</code></td>\n\
<td><code>VT_I4</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>Float</code></td>\n\
<td><code>VT_R4</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>Boolean</code></td>\n\
<td><code>VT_BOOL</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>Color</code></td>\n\
<td><code>VT_I4</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>String</code></td>\n\
<td><code>VT_BSTR</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>Undefined</code></td>\n\
<td><code>VT_EMPTY</code></td>\n\
</tr>\n\
<tr>\n\
<td><code>OLEObject</code></td>\n\
<td><code>VT_DISPATCH</code></td>\n\
</tr>\n\
</tbody></table>\n\
<p>The following information is taken from the document &quot;Accessing Data with ADO&quot; from MSDN (<strong>ms-help://MS.MSDNQTR.2003OCT.1033/iisref/htm/AccessingDatawithADO.htm</strong>).</p>\n\
<p><strong>The following table lists OLE DB connection strings for several common data sources:</strong></p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Data Source</th>\n\
<th>OLE DB Connection String</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>Microsoft Access</td>\n\
<td>Provider=Microsoft.Jet.OLEDB.4.0;Data Source=<strong>physical path to .mdb file</strong></td>\n\
</tr>\n\
<tr>\n\
<td>Microsoft SQL Server</td>\n\
<td>Provider=SQLOLEDB.1;Data Source=<strong>path to database on server</strong></td>\n\
</tr>\n\
<tr>\n\
<td>Oracle</td>\n\
<td>Provider=MSDAORA.1;Data Source=<strong>path to database on server</strong></td>\n\
</tr>\n\
<tr>\n\
<td>Microsoft Indexing Service</td>\n\
<td>Provider=MSIDXS.1;Data Source=<strong>path to file</strong></td>\n\
</tr>\n\
</tbody></table>\n\
<p>To provide for backward compatibility, the OLE DB Provider for ODBC supports ODBC connection string syntax. The following table lists commonly used ODBC connection strings:</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Data Source Driver</th>\n\
<th>ODBC Connection String</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td>Microsoft Access</td>\n\
<td>Driver={Microsoft Access Driver (<em>.mdb)};DBQ=*</em>physical path to .mdb file**</td>\n\
</tr>\n\
<tr>\n\
<td>SQL Server</td>\n\
<td>DRIVER={SQL Server};SERVER=<strong>path to server</strong></td>\n\
</tr>\n\
<tr>\n\
<td>Oracle</td>\n\
<td>DRIVER={Microsoft ODBC for Oracle};SERVER=<strong>path to server</strong></td>\n\
</tr>\n\
<tr>\n\
<td>Microsoft Excel</td>\n\
<td>Driver={Microsoft Excel Driver (<em>.xls)};DBQ=*</em>physical path to .xls file**; DriverID=278</td>\n\
</tr>\n\
<tr>\n\
<td>Microsoft Excel 97</td>\n\
<td>Driver={Microsoft Excel Driver (<em>.xls)};DBQ=*</em>physical path to .xls file**;DriverID=790</td>\n\
</tr>\n\
<tr>\n\
<td>Paradox</td>\n\
<td>Driver={Microsoft Paradox Driver (<em>.db)};DBQ=*</em>physical path to .db file**;DriverID=26</td>\n\
</tr>\n\
<tr>\n\
<td>Text</td>\n\
<td>Driver={Microsoft Text Driver (<em>.txt;</em>.csv)};DBQ=<strong>physical path to .txt file</strong></td>\n\
</tr>\n\
<tr>\n\
<td>Microsoft Visual FoxPro (with a database container)</td>\n\
<td>Driver= {MicrosoftVisualFoxProDriver};SourceType=DBC;SourceDb=<strong>physical path to .dbc file</strong></td>\n\
</tr>\n\
<tr>\n\
<td>Microsoft Visual FoxPro (without a database container)</td>\n\
<td>Driver= {MicrosoftVisualFoxProDriver};SourceType=DBF;SourceDb=<strong>physical path to .dbf file</strong></td>\n\
</tr>\n\
</tbody></table>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../OLE-Automation.html\">OLE Automation</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";