var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-32FB1A07-6F09-4E85-B7D3-1E9CDE3FBB4C\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Error Messages</h1></div><p>When MAXScript detects a runtime error in your script, it displays an error message and prints diagnostic information to the Listener output pane.</p>\n\
<p>If the error occurs in a controller script or rollout panel script, the error message is displayed in an Alert Box window and is also printed in the Listener output pane.</p>\n\
<p><strong>3ds Max</strong> also features a <a href=\"./../The-MAXScript-Debugger.html\">MAXScript Debugger</a> which provides a more advanced toolset for dealing with errors and monitoring variables.</p>\n\
<h3 id=\"listener-call-stack-trace-back-and-editor-error-highlight\">Listener Call Stack Trace-Back And Editor Error Highlight</h3><p>The diagnostic information is in the form of a <em>call stack trace-back</em> which can help determine the cause and location of the error in your code.</p>\n\
<p>The call stack trace-back shows</p>\n\
<ul>\n\
<li><p>The nesting of called MAXScript functions at the point of error, deepest ones first to outermost ones last;</p>\n\
</li>\n\
<li><p>The filename, position and line number the error occurred at;</p>\n\
</li>\n\
<li><p>The values in all the local variables and function parameters at the levels they are declared or first used.</p>\n\
</li>\n\
</ul>\n\
<p>Note that <code>for</code> loops produce separate entries in the call stack trace-back and include a variable dump for the loop variable and any locals in the loop body. This is because a for loop body represents a <a href=\"./../../MAXScript-Language-Reference/Variables-Assignment-and-Scope/Scope-of-Variables.html\">variable scope</a>.</p>\n\
<p>As a further aid in locating the error, MAXScript highlights the line in which the error occurred in an MAXScript Editor window if the running code was compiled from a source file (any code not entered directly in the Listener). If the source file is not currently open in the <a href=\"./../Accessing-MAXScript/The-MAXScript-Editor-Windows.html\">MAXScript Editor</a>, MAXScript opens the file in a new tab. The line containing the code in which the error occurred is highlighted and scrolled into view.</p>\n\
<p>The error message is either displayed in an Alert Box window or the Listener window. In both cases, the Alert Box or Listener window is brought to the front and the MAXScript Editor window containing the code causing the error is immediately behind.</p>\n\
<p>Compile and runtime error messages reported in the Listener output pane are preceded by the comment symbol &quot;--&quot;, so you can reselect and evaluate code in Listener that might have embedded error messages and not have them cause syntax errors.</p>\n\
<p>In the following figure, an Editor and a Listener window are shown where an error was detected while running the script.</p>\n\
<p>The error occurred in the highlighted line in the Editor window.</p>\n\
<p>Looking at the call stack trace-back, we see the error occurred when the <code>for</code> loop variable <code>i</code> is equal to 100.</p>\n\
<p>In the line in error, the position of the object from array <code>b</code> with index i is being set.</p>\n\
<p>After further investigation, it was found the array <code>b</code> only had 99 elements defined.</p>\n\
<p>Thus element <code>b\\[i\\]</code> contained the value <code>undefined</code> , which resulted in the error as there is no position property associated with <code>undefined</code> .</p>\n\
<p><img src=\"./images/GUID-970A912F-8366-469B-9B93-75922D3E0173-low.gif\" alt=\"\"></p>\n\
<blockquote>\n\
<p>   <strong>ADVANCED INFORMATION ON THIS SPECIFIC ERROR:</strong></p>\n\
<p>   The correct way to avoid this particular error would be to use &#39;for i = 1 to b.count do&#39; to ensure the loop never exceeds the number of objects in b, and also divide by b.count in the line <code>z=360.0*i/b.count</code>.</p>\n\
<p>   Obviously, the above &quot;mistake&quot; was introduced to cause an error for demonstration purposes.</p>\n\
<p>   The script itself tries to place all selected objects in a circle with radius the Radius around the point pos0.</p>\n\
</blockquote>\n\
<h3 id=\"script-controller-errors\">Script Controller Errors</h3><p>In the following figure, an error dialog and script controller dialog are shown.</p>\n\
<p>An error was detected while running the script assigned to a script controller.</p>\n\
<p>This error was caused by the deletion of object Box001, whose position was used directly in the script.</p>\n\
<p><img src=\"./images/GUID-AB5E55EC-78F2-4188-8273-854F90151359-low.gif\" alt=\"\"></p>\n\
<p>The MAXScript Listener shows the following call stack trace-back for this error:</p>\n\
<p><img src=\"./images/GUID-458A6EC5-61FD-43F6-945C-F5AD5C7C466B-low.gif\" alt=\"\"></p>\n\
<blockquote>\n\
<p>   <strong>ADVANCED INFORMATION ON THIS SPECIFIC ERROR:</strong></p>\n\
<p>   Since <strong>3ds Max 8</strong>, the above way of accessing object properties is considered obsolete and a Bad Practice exactly because it can cause a large number of errors when a referenced object is deleted or cannot be found for other reasons (for example when an object with a Script Controller is brought into a scene via XRef).</p>\n\
<p>   The correct approach to avoid this specific kind of error would be to use a Variable assigned the Position track of Box001. Deleting the Box001 would not cause an error in the controller. As an added bonus, the controlled object would update its position when Box001 is moved along Z automatically.</p>\n\
<p>   Please see <a href=\"./../../3ds-Max-Objects-and-Interfaces/Animation-Controllers/Controller-Types/Script-Controllers/Script-Controllers-1.html\">Script Controllers</a> for details.</p>\n\
</blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../General-MAXScript-Topics.html\">General MAXScript Topics</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../The-MAXScript-Debugger.html\">The MAXScript Debugger</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";