var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-B48DA525-F64B-480C-A6E1-3A05D09007F7\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Function Parameters</h1></div><p>The parameters to a function are implicitly declared as local variables at the head of the function body, and are initialized with the arguments given by the caller. The parameters have a scope that extends to the end of the function body <code>&lt;expr&gt;</code> . They can be assigned to and hidden by nested local variables with the same name as can other local variables.</p>\n\
<p>Keyword parameters can be given default values in the function definition, implicitly making them optional to the caller. If the caller does not supply one of the keyword arguments, it is initialized to the default value upon function entry. The default values are specified after the <code>&apos;:&apos;</code> (colon).</p>\n\
<p>If a keyword parameter is defined with no default value and is not supplied by the caller, it is initialized to the special value <code>unsupplied</code> . You can check for this value in the body of your function and handle missing required keyword argument errors accordingly.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn foo a b: c:&quot;default c&quot;  =\n\
   (\n\
\n\
       if b == unsupplied then format &quot;Where&apos;s the b: argument??&quot;\n\
       else format &quot;a:% b:% c:%&quot; a b c\n\
   )\n\
\n\
   foo &quot;A&quot; b:&quot;B&quot;\n\
   --&gt; a:A b:B c:default c\n\
   foo &quot;A&quot; b:&quot;B&quot; c:&quot;C&quot;\n\
   --&gt; a:A b:B c:C\n\
   foo &quot;A&quot; c:&quot;C&quot;\n\
   --&gt; Where&apos;s the b: argument??\n\
</code></pre></blockquote>\n\
<p>For more information about keyword arguments and default values, see <a href=\"./../Names-Literal-Constants-and/Simple-Expressions/Function-Calls/Positional-and-Keyword-Arguments.html\">Positional and Keyword Arguments</a>.</p>\n\
<p>As described in <a href=\"./../Variables-Assignment-and-Scope/Reference-Assignment.html\">Reference Assignment</a>, MAXScript uses an assignment called <em>reference assignment</em>. In a function, each parameter variable contains a reference to the value passed to the function by the caller. Assigning to a parameter variable places a new reference in the parameter variable, and has no effect on the variable values in the caller (it is an entirely local action). An exception to this is when the parameter value is a compound value such as, a 3D Point, string, or array, and you assign a new value to one of the components of the compound value. In this case, the variable&apos;s reference does not change, and the changed compound value is still referenced by the variable in the caller.</p>\n\
<p>While in some cases you might want a function to return multiple values, it is typically not a good practice to do this by assigning to the components of a parameter variable compound value. To manipulate a compound value passed as a parameter, a compound value needs to be created in the caller and passed to the function, which can give results as shown in this example:</p>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn getXYZset val =\n\
   (\n\
   val.x=random -100 100\n\
   val.y=random 100 100\n\
   val.z=random val.x val.y\n\
   )\n\
   (\n\
   v1=[0,0,0]\n\
   v4=v1\n\
   getXYZSet v1\n\
   format &quot;v1= %; v4= %\\n&quot; v1 v4\n\
   getXYZSet v4\n\
   format &quot;v1= %; v4= %\\n&quot;v1 v4\n\
   )\n\
</code></pre><p>   <strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   getXYZset() -- result lines 1 to 5\n\
   v1= [-84,100,78.1224]; v4= [-84,100,78.1224] -- output line 10\n\
   v1= [10,100,18.6575]; v4= [10,100,18.6575] -- output line 12\n\
   OK -- result lines 6 to 12\n\
</code></pre></blockquote>\n\
<p>Note that it appears the value for both variables <code>v1</code> and <code>v4</code> are set in the calls to <code>getXYZset</code> , even though only one variable is being passed. The reason for this is in line 8, variable <code>v4</code> is initialized to the value of <code>v1</code> . What actually occurs is that in line 7, a reference to a Point3 value of [0,0,0] is created and this reference is stored in <code>v1</code> . In line 8, that same reference is retrieved from <code>v1</code> and assigned to <code>v4</code> . Rather than storing two different values, both <code>v1</code> and <code>v4</code> reference the same value. When you set the components of the compound value in <code>getXYZset</code> , the reference to the Point3 value is not being changed, and both variables still reference that value.</p>\n\
<p>The proper way to return multiple values from a function is to have the return value of the function be an array or a structure. For example,</p>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn readDataFile filename =\n\
   (\n\
   f=openfile filename\n\
   data=#()\n\
   avg=0\n\
   nVals=0\n\
   while not eof f do\n\
   (\n\
   val = readvalue f\n\
   append data val\n\
   avg += val\n\
   nVals += 1\n\
   )\n\
   close f\n\
   #(data,(avg/nVals))\n\
   )\n\
   result=readDataFile &quot;c:\\\\datafiles\\\\run1.dat&quot;\n\
   data=result[1]\n\
   avg=result[2]\n\
</code></pre></blockquote>\n\
<p>Normally, if you are writing a function that is operating on a compound value that is passed as a parameter, you must make a <code>copy</code> of the value. This prevents the value in the caller from inadvertently being changed. For example,</p>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn uppercase instring =\n\
   (\n\
   local upper, lower, outstring\n\
   upper=&quot;ABCDEFGHIJKLMNOPQRSTUVWXYZ&quot;\n\
   lower=&quot;abcdefghijklmnopqrstuvwxyz&quot;\n\
   outstring=copy instring -- operate on copy of instring\n\
   for i=1 to outstring.count do\n\
   (\n\
   j=findString lower outstring[i]\n\
   if j != undefined do outstring[i]=upper[j]\n\
   )\n\
   return outstring\n\
   )\n\
</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Creating-Functions.html\">Creating Functions</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";