var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-7B851ED2-B59C-45E7-97BF-16B0A5C02971\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Function Variables</h1></div><p>When you define a function, the following events occur:</p>\n\
<ul>\n\
<li><p>The function definition is evaluated, creating a function value.</p>\n\
</li>\n\
<li><p>The function <code>&lt;name&gt;</code> is used to declare a new variable with that name.</p>\n\
</li>\n\
<li><p>The function value is placed into that variable.</p>\n\
</li>\n\
<li><p>The function value is returned as the result of the function definition expression.</p>\n\
</li>\n\
</ul>\n\
<p>Because the function itself is a value stored in a variable, you not only can call the function through the function name variable. By simply referring the function&apos;s variable you can also assign the function value to another variable, store it in an array, or pass it as an argument to another function.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   surface_functions=#()\n\
   fn build_math_mesh x = print &quot;build math mesh&quot;\n\
\n\
   -- some 3D surface functions\n\
   fn saddle x y = sin x * sin y\n\
   fn sombrero x y = cos (x^2 + y^2) / (1 + (x^2 + y^2))\n\
   print (saddle 1 1) -- try one out\n\
   surface_functions[1] = saddle -- store the functions in an array\n\
   surface_functions[2] = sombrero\n\
   -- call one of them from within the array:\n\
   z = (surface_functions[1]) 10 10\n\
   -- pass one in to a higher-order function:\n\
   build_math_mesh sombrero\n\
</code></pre></blockquote>\n\
<p>The scope of a function variable is the current MAXScript scope context. The variables used inside a function have a local scope, unless the variable was already defined at a higher scope or is declared as global in the function. An exception to this is the function parameter variables, which always have a local scope.</p>\n\
<p>When writing scripts, it is <strong>good programming practice</strong> to <strong>explicitly declare your local and global variables</strong>. Implicit declaration is provided as a short-hand, typically used when working in the Listener interactively or developing short scripts. When developing extended scripts, explicitly declaring variables can reduce errors and improve readability of the code. It is also recommend that you declare as local all variables unless you really want them to be global variables. The reasons for this are described in <a href=\"./../Variables-Assignment-and-Scope/Scope-of-Variables.html\">Scope of Variables</a>.</p>\n\
<p>If a function variable is assigned to another variable, a reference to the function value is stored in that variable. Thus while a function variable may no longer be in scope, the function itself still is if the variable it is assigned to has a scope that extends outside the current MAXScript scope context.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn funcA which =\n\
   (\n\
   if which == 1 then\n\
   fn saddle x y = sin x * sin y\n\
   else\n\
   fn sombrero x y = cos (x^2 + y^2) / (1 + (x^2 + y^2))\n\
   )\n\
   myfunc=funcA 2\n\
   z=myfunc 10 10\n\
</code></pre><p>   In the above example, the scope of variables <code>saddle</code> and <code>sombrero</code> is local to <code>funcA</code> , and the variables are out of scope outside the function. The return value of <code>funcA</code> is either the function value <code>saddle()</code> or <code>sombrero()</code> , and is assigned to variable <code>myfunc</code> . As also can be seen in this example, you can define a function within any expression.</p>\n\
</blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Creating-Functions.html\">Creating Functions</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Variables-Assignment-and-Scope/Scope-of-Variables.html\">Scope of Variables</a></li>\n\
          <li><a class=\"link\" href=\"../Names-Literal-Constants-and/Simple-Expressions/Function-Calls.html\">Function Calls</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";