var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-7C91D285-5683-4606-9F7C-B8D3A7CA508B\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Node Event System</h1></div><p>The <strong>Node Event System</strong> available in <strong>3ds Max 2009 and higher</strong> lets you define a scripted function that is triggered in response to any of various node-related scene events. The events map one-to-one with the event methods of class <strong>INodeEventCallback</strong> in the <strong>3ds Max SDK</strong>. For detailed information on each, read the methods comments for class <strong>INodeEventCallback</strong> in <strong>maxsdk\\include\\ISceneEventManager.h</strong></p>\n\
<p>The Node Event System is similar to the <a href=\"General-Event-Callback-Mechanism.html\">General Event Callback Mechanism</a>. It operates as a layer over the General Event Callback system, but also receives some node messages which the older system cannot handle, like geometry, topology and mapping changes. This system will also catch events which happen as a result of undo/redo, while the older system sometimes would not.</p>\n\
<p>Thus, if you wish to catch nodes-related events and changes to nodes, the Node Event System is your best option.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> The Node Event System is <strong>inappropriate</strong> for catching any events except those specifically related to <strong>scene nodes</strong>. It is mainly intended to be used with a UI (such as a scripted plug-in). The system is timer based, so it requires the Windows message loop to be running, which means its use in a script without a UI can be problematic.</div>\n\
<p><strong>Contructor:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;callbackItem&gt;NodeEventCallback [mouseUp:&lt;boolean&gt;] [delay:&lt;integer&gt;] \\\n\
[enabled:&lt;boolean&gt;] [polling:&lt;boolean&gt;] \\\n\
[callbackBegin:&lt;fn&gt;] [all:&lt;fn&gt;] \\\n\
[added:&lt;fn&gt;] [deleted:&lt;fn&gt;] \\\n\
[linkChanged:&lt;fn&gt;] [layerChanged:&lt;fn&gt;]\\\n\
[groupChanged:&lt;fn&gt;] [hierarchyOtherEvent:&lt;fn&gt;] \\\n\
[modelStructured:&lt;fn&gt;] [geometryChanged:&lt;fn&gt;] \\\n\
[topologyChanged:&lt;fn&gt;] [mappingChanged:&lt;fn&gt;] \\\n\
[extentionChannelChanged:&lt;fn&gt;] [modelOtherEvent:&lt;fn&gt;] \\\n\
[materialStructured:&lt;fn&gt;] [materialOtherEvent:&lt;fn&gt;] \\\n\
[controllerStructured:&lt;fn&gt;] [controllerOtherEvent:&lt;fn&gt;] \\\n\
[nameChanged:&lt;fn&gt;] \\\n\
[wireColorChanged:&lt;fn&gt;] [renderPropertiesChanged:&lt;fn&gt;] \\\n\
[displayPropertiesChanged:&lt;fn&gt;] [userPropertiesChanged:&lt;fn&gt;] \\\n\
[propertiesOtherEvent:&lt;fn&gt;] \\\n\
[subobjectSelectionChanged:&lt;fn&gt;] [selectionChanged:&lt;fn&gt;] \\\n\
[hideChanged:&lt;fn&gt;] [freezeChanged:&lt;fn&gt;] \\\n\
[displayOtherEvent:&lt;fn&gt;] [callbackEnd:&lt;fn&gt;]      </code></pre><p>The example script below prints a message to the MAXScript Listener when any message is received by registering a function to the optional keyword argument <code>all:</code></p>\n\
<p>The script also displays a message box when the <code>linkChanged:</code> or <code>layerChanged:</code> event occurs, that is, whenever one node is linked or unlinked from another node, or moved to a different layer.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   --Define a callback function to handle linking and layer changes:\n\
   fn CallbackFn1 ev nd = (\n\
   messageBox (&quot;Event Detected: Event &quot;+(ev as string)+ \\\n\
   &quot;, Nodes &quot; + (nd as string))\n\
   )\n\
   --Define a callback function to handle ALL messages:\n\
   fn CallbackFn2 ev nd = (\n\
   format &quot;Event Detected: Event%, Nodes %\\n&quot; ev nd )\n\
   --Register callbacks for link changes, layer changes using the\n\
   --first function and for all changes using the second function.\n\
   --Act only when mouse is up and there have been no other events\n\
   --within one second (1000 milliseconds):\n\
   callbackItem = NodeEventCallback mouseUp:true delay:1000 linkChanged:CallbackFn1 layerChanged:CallbackFn1 all:CallbackFn2\n\
</code></pre></blockquote>\n\
<p><strong>Destructor:</strong></p>\n\
<p>When the <code>NodeEventCallback()</code> constructor is called, an instance of the class is created and returned as the result of the call.</p>\n\
<p>To release this instance and thus remove the callback, the value of the variable holding the handler to the instance has to be set to undefined and the garbage collector must be called to reclaim the memory and effectively destroy the callback.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   --Continuing the previous example where the variable\n\
   --callbackItem was used to store the callback instance,\n\
   callbackItem = undefined\n\
   gc light:true\n\
</code></pre></blockquote>\n\
<h2 id=\"handling-optional-keyword-parameters-\">Handling optional keyword parameters:</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">mouseup: </code></pre><p>If the optional keyword argument <code>mouseUp:</code> is set to <code>true</code>, the callback will only be triggered when the mouse button is released.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">delay: </code></pre><p>When the optional keyword argument <code>delay:</code> is supplied (as time in milliseconds), the callback is only triggered if the given time passes with no events, that is, only after the system is idle for the specified time.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">polling: </code></pre><p>When the optional keyword argument <code>polling:</code> is set to true, the pending events for the <code>NodeEventCallbackValue</code> can be fired by calling <code>triggerNodeEventCallback()</code>. Available in <strong>3ds Max 2019.3 Update and higher</strong>. The <strong>polling</strong> argument cannot be used if either <strong>mouseUp</strong> or <strong>delay</strong> are specified.</p>\n\
<h2 id=\"callback-function-optional-keyword-parameters-\">Callback Function optional keyword parameters:</h2><p>One or more of the other optional keyword arguments can be supplied with pre-defined MAXScript Functions to be executed if a callback of the specified type is found in the message received by the Node Event System.</p>\n\
\n\
<div class=\"callout callout-important\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-exclamation-triangle\" aria-hidden=\"true\"></i>Important:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p>   </p>\n\
<p>   The callback functions <strong>always</strong> take <strong>two parameters</strong>: the first is the event name that triggered the function call, the second is a list of node <a href=\"./../../3ds-Max-Objects-and-Interfaces/Identifying-and-Accessing/AnimHandle-System.html\">AnimHandles</a>.</p>\n\
<p>   The <code>GetAnimByHandle()</code> method must be used to get the node itself.</p>\n\
<p>   If the specified function does not expect two arguments, an error will be thrown. For example, if the function ObjFn is passed but it does not provide any arguments, the error will look like</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   &gt;&gt; MAXScript NodeEventCallback Exception: -- Argument count error: ObjFn wanted 0, got 2 &lt;&lt;\n\
</code></pre><p>   From the text of the error one could incorrectly assume that the function was correct and the callback was wrong, but this is not the case.</p>\n\
<p>   The callback always passes the two arguments, but if the function is unable to accept them, the error message simply reflects that fact as it has no knowledge what called the function and why it passed two parameters to it.</p>\n\
\n\
</div></div>\n\
\n\
<h3 id=\"callback-batch-handling-\">Callback Batch Handling:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">callbackBegin:   \n\
callbackEnd: </code></pre><p>When a callback is triggered, messages for all queued events will be sent in <strong>one batch</strong>.</p>\n\
<p>The <code>callbackBegin:</code> message indicates the start of a batch, while <code>callbackEnd:</code> indicates the end of a batch.</p>\n\
<p>Registering functions using these two optional keyword arguments lets you get notification of the beginning and the end of a single message batch.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">all: </code></pre><p>The supplied function will be called whenever ANY of the node-related messages listed below has been received. Note that this function is called in addition to any event-specific functions that have been registered. The <code>all</code> function is called first.</p>\n\
<h3 id=\"node-creation-and-removal-callbacks-\">Node Creation and Removal Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">added: </code></pre><p>The supplied function will be called whenever a node has been added to the scene.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">deleted: </code></pre><p>The supplied function will be called whenever a node has been deleted from the scene.</p>\n\
<h3 id=\"node-hierarchy-layer-and-group-callbacks-\">Node Hierarchy, Layer and Group Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">linkChanged: </code></pre><p>The supplied function will be called whenever the node is linked to or unlinked from another node.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">layerChanged: </code></pre><p>The supplied function will be called whenever the node has moved to another layer.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">groupChanged: </code></pre><p>The supplied function will be called whenever the node has been added or removed from a group.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">hierarchyOtherEvent: </code></pre><p>The supplied function will be called whenever the node&apos;s hierarchy has been changed in some other way.</p>\n\
<h3 id=\"node-model-geometry-topology-and-mapping-callbacks-\">Node Model, Geometry, Topology and Mapping Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">modelStructured: </code></pre><p>The supplied function will be called whenever the model&apos;s structure has been modified.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">geometryChanged: </code></pre><p>The supplied function will be called whenever the geometry of the node has changed, for example a vertex has been moved.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">topologyChanged: </code></pre><p>The supplied function will be called whenever the topology of the node&apos;s geometry has changed, for example an edge has been added or removed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">mappingChanged: </code></pre><p>The supplied function will be called whenever the mapping of the node&apos;s geometry has changed, for example UV coordinates have been assigned or modified.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">extentionChannelChanged: </code></pre><p>The supplied function will be called whenever the a non-mapping channel of the node has been changed, for example Vertex Alpha, Soft-Selection and so on.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">modelOtherEvent: </code></pre><p>The supplied function will be called whenever the node&apos;s model has been changed in some other way.</p>\n\
<h3 id=\"node-material-callbacks-\">Node Material Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">materialStructured: </code></pre><p>The supplied function will be called whenever a material has been added or removed from the node.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">materialOtherEvent: </code></pre><p>The supplied function will be called whenever the node&apos;s material has been changed.</p>\n\
<h3 id=\"node-controller-callbacks-\">Node Controller Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">controllerStructured: </code></pre><p>The supplied function will be called whenever a controller has been assigned or removed to a SubAnim track of the node.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">controllerOtherEvent: </code></pre><p>The supplied function will be called whenever a node&apos;s controller has been changed in another way.</p>\n\
<h3 id=\"node-name-and-properties-callbacks-\">Node Name and Properties Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">nameChanged: </code></pre><p>The supplied function will be called whenever the name of the node has been changed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">wireColorChanged: </code></pre><p>The supplied function will be called whenever the wireframe color of the node has been changed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">renderPropertiesChanged: </code></pre><p>The supplied function will be called whenever the Render Properties of the node has been changed, including Renderable, Visible To Camera, Visible To Reflections and so on.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">displayPropertiesChanged: </code></pre><p>The supplied function will be called whenever a display property of the node has been changed, for example &quot;Display As Box&quot;, &quot;Show Vertex Ticks&quot;, &quot;Backface Culling&quot; and so on.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">userPropertiesChanged: </code></pre><p>The supplied function will be called whenever the User Properties buffer of the node has been changed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">propertiesOtherEvent: </code></pre><p>The supplied function will be called whenever any other properties of a node have been changed.</p>\n\
<h3 id=\"node-selection-callbacks-\">Node Selection Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">subobjectSelectionChanged: </code></pre><p>The supplied function will be called whenever the sub-object selection of a node has changed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">selectionChanged: </code></pre><p>The supplied function will be called whenever the selected state of the node has changed, for example a node has been selected or deselected.</p>\n\
<h3 id=\"node-display-callbacks-\">Node Display Callbacks:</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">hideChanged: </code></pre><p>The supplied function will be called whenever the Hidden flag of a node has changed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">freezeChanged: </code></pre><p>The supplied function will be called whenever the Frozen flag of a node has changed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">displayOtherEvent: </code></pre><p>The supplied function will be called whenever other display flags of a node have changed.</p>\n\
<h2 id=\"related-functions\">Related Functions</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">triggerNodeEventCallback &lt;NodeEventCallbackValue&gt;</code></pre><p>Triggers the pending events for the specified NodeEventCallbackValue. Available in <strong>3ds Max 2019.3 Update and higher</strong>. This function works for <code>NodeEventCallbackValues</code> with <code>polling</code> specified as true.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;array&gt; getNodeEventCallbacks </code></pre><p>Gets an array of all registered NodeEventCallback values in the current scene. Available in <strong>3ds Max 2019.3 Update and higher</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">showNodeEventCallbacks [asArray:&lt;boolean&gt;] [to:&lt;stream&gt;] [</code></pre><p>Returns information about all registered NodeEventCallback values in the current scene, including the <strong>enabled</strong>, <strong>polling</strong>, <strong>mouseUp</strong>, and <strong>delay</strong> settings, the eventIds registered and the functions associated with those events, and the script file name that created the NodeEventCallback. Available in <strong>3ds Max 2019.3 Update and higher</strong>.</p>\n\
<p>If the optional <strong>asArray</strong> keyword parameter is specified as true, this information is returned as an array of strings, one for each NodeEventCallback. Otherwise it is a single string.</p>\n\
<p>If the optional <strong>to</strong> keyword parameter is specified, the output is written to the specified stream.</p>\n\
<h3 id=\"example\">Example</h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">poll_event_count = 0\n\
\n\
fn runExample = (\n\
-- reset the scene\n\
    resetMaxFile #noprompt --reset the scene\n\
    gc() -- run gc again to clean out previous runs\n\
\n\
    --Define a callback function to handle ALL messages:\n\
    fn MyCallback ev nd = (format &quot;Event Detected: Event%, Nodes %\\n&quot; ev nd )\n\
\n\
    fn MyPollCallback ev nd =     (\n\
        format &quot;Event Detected: Event%, Nodes %\\n&quot; ev nd\n\
        poll_event_count += 1 )\n\
\n\
    -- Create callback item\n\
    callbackItem = NodeEventCallback added:MyCallback \n\
    pollCallBackItem = NodeEventCallback added:MyPollCallback    \n\
\n\
    -- add something, which will trigger the first callback, but not the second:\n\
    t = teapot()\n\
\n\
    -- this will still be 0, because polling is on:\n\
    format &quot;poll events: % \\n&quot; poll_event_count\n\
\n\
    -- trigger polling callback\n\
    triggerNodeEventCallback pollCallBackItem\n\
\n\
    -- node event system is timer based, and requires\n\
    -- message pump system.  This will give the system\n\
    -- time to &quot;catch up&quot;, since we do not have a UI\n\
    windows.processpostedmessages();sleep .1\n\
    windows.processpostedmessages()\n\
\n\
    -- did it trigger?\n\
    format &quot;poll events: % \\n&quot; poll_event_count\n\
\n\
    showNodeEvents = showNodeEventCallbacks asArray:true\n\
    format &quot;showNodeEventCallbacks : % \\n&quot; showNodeEvents\n\
\n\
    getNodeEvents = getNodeEventCallbacks()\n\
    format &quot;Number of event callbacks: %\\n&quot; getNodeEvents.count\n\
    for i = 1 to getNodeEvents.count do format &quot;Callback % : % \\n&quot; i getNodeEvents[i]\n\
\n\
    -- delete the callback items, and garbage collect:\n\
    callbackItem = pollCallbackItem = undefined\n\
    gc()\n\
\n\
)\n\
\n\
runExample()\n\
\n\
--&gt; Output:\n\
runExample()\n\
poll events: 0 \n\
Event Detected: Event#added, Nodes #(6772)\n\
Event Detected: Event#added, Nodes #(6772)\n\
poll events: 1 \n\
showNodeEventCallbacks : #(#(NodeEventCallback&lt;0x000001A535A18210&gt;, #added, &quot;MyPollCallback&quot;, \\\n\
 &quot;D:\\My Documents\\3ds Max\\scripts\\NodeEventCallbacksEx.ms&quot;), #(NodeEventCallback&lt;0x000001A535A229F4&gt;,\\\n\
 #added, &quot;MyCallback&quot;, &quot;D:\\My Documents\\3ds Max\\scripts\\NodeEventCallbacksEx.ms&quot;)) \n\
Number of event callbacks: 2\n\
Callback 1 : NodeEventCallback&lt;0x000001A535A18210&gt; \n\
Callback 2 : NodeEventCallback&lt;0x000001A535A229F4&gt; \n\
482872L\n\
OK</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"General-Event-Callback-Mechanism.html\">General Event Callback Mechanism</a></li>\n\
          <li><a class=\"link\" href=\"Change-Handlers-and-When.html\">Change Handlers and When Constructs</a></li>\n\
          <li><a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";