var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-C0BFFE27-BB59-4265-8F70-BFE8A636F7CB\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Scripted SimpleObject Plug-ins</h1></div><p>A SimpleObject is a kind of geometry primitive that is defined by a tri-mesh such as, a box, sphere, cone, and so on. In other words, a simpleObject plugin is similar to all the Standard and Extended primitives in <strong>3ds Max</strong>.</p>\n\
<p>In scripting a SimpleObject plug-in, you need to supply a handler for building this mesh and <strong>3ds Max</strong> automatically handles the scene display, hit-testing, ray intersection, rendering, modifier applicability, and others.</p>\n\
<p>A scripted SimpleObject plug-in is declared by specifying the <code>&lt;superclass&gt;</code> as <code>simpleObject</code> .</p>\n\
<p>Scripted SimpleObject plug-ins <strong>require</strong> a <code>create</code> tool.</p>\n\
<p>A SimpleObject plug-in must not perform any scene related actions such as, creating scene nodes or building modifier stacks. It must only adjust its parameters in the mouse tool handlers and construct a mesh in the <code>buildMesh</code> handler. Attempting to create scene nodes or applying modifiers in a SimpleObject plug-in will cause all kinds off strange interactions with the in-progress scene node creation.</p>\n\
<p>A SimpleObject plug-in has a predefined local variable <code>mesh</code> . This variable contains the underlying mesh of the object being created. The <code>mesh</code> local variable is accessible in any of the plug-in&apos;s handlers, but is typically only built in the <code>buildMesh</code> handler. You can either modify the existing TriMesh value in place using the TriMesh methods, or construct a new TriMesh value and assign it to the <code>mesh</code> plug-in local variable. The TriMesh methods and properties are described in <a href=\"./../../../3ds-Max-Objects-and-Interfaces/Editable-Meshes-Splines-Patches/Editable_Mesh-and-TriMesh/Editable_Mesh-GeometryClass-and.html\">Editable Mesh</a>.</p>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   plugin simpleObject tower_plugin_def\n\
   name:&quot;Tower&quot;\n\
   classID:#(145345,543211)\n\
   category:&quot;Scripted Primitives&quot;\n\
   (\n\
     parameters main rollout:params\n\
     (\n\
       height type:#worldUnits ui:height default:0\n\
       width type:#worldUnits ui:width default:0\n\
       depth type:#worldUnits ui:depth default:0\n\
     )\n\
     rollout params &quot;Two Faces Parameters&quot;\n\
     (\n\
       spinner height &quot;Height&quot; type:#worldunits range:[-1000,1000,0]\n\
       spinner width &quot;Width&quot; type:#worldunits range:[-1000,1000,0]\n\
       spinner depth &quot;Depth&quot; type:#worldunits range:[-1000,1000,0]\n\
     )\n\
     on buildMesh do\n\
     (\n\
       setMesh mesh \\\n\
       verts:#([0,0,0],[width,0,0],[width,depth,0],[0,depth,0]) \\\n\
       faces:#([3,2,1], [1,4,3])\n\
       extrudeFace mesh #(1,2) (height * 0.5) 40 dir:#common\n\
       extrudeFace mesh #(1,2) (height * 0.5) 50 dir:#common\n\
     )\n\
     tool create\n\
     (\n\
       on mousePoint click do\n\
         case click of\n\
         (\n\
           1: nodeTM.translation = gridPoint\n\
           3: #stop\n\
          )\n\
       on mouseMove click do\n\
         case click of\n\
         (\n\
           2: (width = gridDist.x; depth = gridDist.y)\n\
           3: height = gridDist.z\n\
         )\n\
     )\n\
   )\n\
</code></pre></blockquote>\n\
<p>In this script a new primitive called Tower is defined. It constructs a very simple tower form; the first drag sets the base and the second drag sets the height. It contains three animatable rollout parameters: <code>height</code> , <code>width</code> , and <code>depth</code> , which set the object&apos;s basic bounds. The key components here are the <code>create</code> tool and the <code>on buildMesh</code> handler, and they work together in a simple way. The <code>create</code> tool sets the values of the parameters and the <code>on buildMesh</code> handler constructs a mesh based on the parameter values.</p>\n\
<p>The <code>create</code> tool can also access and set the position of the node that is being created to contain the new object through the Matrix3 value in the pre-defined mouse tool local variable <code>nodeTM</code> . In this case, the position portion of the node&apos;s TM is set to the initial mouse down world position. The <code>on mouseMove</code> handler sets the width and depth during click 2 and the height during click 3.</p>\n\
<p>The <code>on buildMesh</code> handler constructs the desired mesh in the TriMesh value in the pre-defined plug-in local variable <code>mesh</code> . Typically, it does this by building the mesh from scratch each time the handler is called. In the above example, the mesh is initially set to a simple two-face rectangular plane (using <code>setMesh()</code> ), and then the two faces are extruded up and scaled-down twice to get the simple tower.</p>\n\
<p>The <code>mesh</code> plug-in local variable is accessible in any of the plug-in&apos;s handlers, but is typically built only in the <code>on buildMesh</code> handler. You can either modify the existing TriMesh value in place using the TriMesh methods, or construct a new TriMesh value and assign it to the <code>mesh</code> plug-in local variable. The TriMesh created must be valid or a <strong>3ds Max</strong> crash might occur. All face, vertex, material ID, and texture vertex arrays allocated must be filled in the handler.</p>\n\
<p>There are three additional event handlers that might be implemented for a scripted SimpleObject:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on OKtoDisplay do &lt;boolean_expr&gt;</code></pre><p>If the <code>OKtoDisplay</code> event handler is implemented, <code>&lt;boolean_expr&gt;</code> must return <code>true</code> or <code>false</code> depending on whether it is OK to display the current mesh. This is useful in situations where a mesh might be in some degenerate state for certain parameter settings and must not be displayed in the viewports. The default implementation is <code>true</code> . Note that empty meshes are OK to display.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on hasUVW do &lt;boolean_expr&gt;</code></pre><p>If the <code>hasUVW</code> event handler is implemented, <code>&lt;boolean_expr&gt;</code> must return <code>true</code> or <code>false</code> depending on whether UVW coordinates are present on the mesh. In many primitive objects, a Generate Mapping Coords checkbox is provided for the user to control UVW coordinate generation and the <code>hasUVW</code> handler expression returns the state of this checkbox. The default implementation returns <code>true</code> or <code>false</code> depending on whether the mesh has texture vertices or not. At present, only a single map channel is supported.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on setGenUVW &lt;onOff&gt; do &lt;expr&gt;</code></pre><p>The <code>setGenUVW</code> event handler is called when <strong>3ds Max</strong> wants the plug-in to automatically generate mapping coordinates. This happens when you first render an object that has a material applied, but not mapping coordinates. It is called with a switch argument <code>&lt;onOff&gt;</code> , which is <code>true</code> or <code>false</code> to turn on or turn off the mapping coordinates. If your plug-in is managing mapping coordinates, it must implement this handler and generate mapping coordinates when called with a <code>true</code> argument.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   plugin simpleObject TexturePlane\n\
   name:&quot;TexturePlane&quot;\n\
   classID:#(0x19a75166, 0x83e3a1c7)\n\
   category:&quot;MAXScript Examples&quot;\n\
   (\n\
   parameters main rollout:params\n\
   (\n\
     Size type:#worldUnits ui:size default:0\n\
     genMapCoords type:#boolean ui:gen_Map_Coords default:false animatable:false\n\
     u_tile type:#float default:1.0 ui:u_step\n\
     v_tile type:#float default:1.0 ui:v_step\n\
   )\n\
   rollout params &quot;TexturePlane&quot;\n\
   (\n\
     group &quot;Geometry&quot;\n\
     (\n\
       spinner size &quot;Size:&quot; type:#worldunits range:[0,1000,1]\n\
     )\n\
     group &quot;Mapping&quot;\n\
     (\n\
       spinner u_step &quot;U Tiling&quot; scale:0.01\n\
       spinner v_step &quot;V Tiling&quot; scale:0.01\n\
       checkbox gen_Map_Coords &quot;Generate Mapping Coords.&quot;\n\
     )\n\
   ) --end roll\n\
\n\
   --When Max wants to know whether the object has UV coords,\n\
   --return the value stored in genMapCoords to inform it of\n\
   --the current UV sate of the plug-in.\n\
\n\
   on hasUVW do genMapCoords \n\
\n\
   --When 3ds Max tells the plug-in to turn on UV coordinates \n\
   --(for example when applying a texture \n\
   --with &quot;Show Map In Viewport&quot; checked), \n\
   --this handler will automagically check the checkbutton \n\
   --connected to our parameter called genMapCoords. \n\
   --Also, when the user changes the checkbox, \n\
   --the variable will tell the builMesh to generate coordinates. \n\
   --Note that the handler will override the manual settings, \n\
   --for example if a texture has &quot;Show Map In Viewport&quot; enabled, \n\
   --you cannot actually uncheck the &quot;Generate Mapping Coords.&quot; \n\
   --checkbox with the mouse! \n\
\n\
   on setGenUVW bool do genMapCoords = bool \n\
\n\
   on buildMesh do ( \n\
     --Generate a simple Quad \n\
     local vertex_array = #() \n\
     append vertex_array [-size/2,-size/2,0] \n\
     append vertex_array [size/2,-size/2,0] \n\
     append vertex_array [size/2,size/2,0] \n\
     append vertex_array [-size/2,size/2,0] \n\
     face_array = #( [1,2,3], [1,3,4] ) \n\
     setMesh mesh verts:vertex_array faces:face_array \n\
     --If the checkbox was enabled and we want \n\
     --coordinates to be generated... \n\
\n\
     if genMapCoords then ( --define the texture vertices \n\
       local uvw_array = #() append uvw_array [0.0,0.0,0.0] append uvw_array [u_tile ,0.0,0.0] append uvw_array [u_tile ,v_tile ,0.0] append uvw_array [0.0,v_tile ,0.0] \n\
       --set the number of vertices \n\
       setNumTVerts mesh uvw_array.count \n\
       --set all vertices \n\
       for v = 1 to uvw_array.count do setTVert mesh v uvw_array[v] \n\
       --build texture faces \n\
       buildTVFaces mesh false \n\
       --set all texture faces (in this case using the mesh faces \n\
       --since we happen to have a one-to-one correspondence \n\
       --between vertices and texture vertices.) \n\
       --This is not always the case though... \n\
       for f = 1 to face_array.count do setTVFace mesh f face_array[f] ) \n\
   )--end on buildMesh \n\
   tool create \n\
   ( \n\
     on mousePoint click do \n\
     case click of \n\
     ( \n\
       1: ( nodeTM.translation = gridPoint ) \n\
       2: ( #stop) \n\
     ) \n\
     on mouseMove click do \n\
     case click of \n\
     ( \n\
       2: (Size= abs gridDist.x) \n\
     ) \n\
     ) \n\
   )--end plugin \n\
</code></pre></blockquote>\n\
<p>Here is another example that creates a mesh by first building other temporary objects and using mesh booleans to build the final mesh:</p>\n\
<blockquote>\n\
<p>   <strong>SCRIPT</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   plugin simpleObject squareTube\n\
   name:&quot;SquareTube&quot;\n\
   classID:#(63445,55332)\n\
   category:&quot;Scripted Primitives&quot;\n\
   (\n\
   local box1, box2\n\
   parameters main rollout:params\n\
   (\n\
     length type:#worldUnits ui:length default:1E-3\n\
     width type:#worldUnits ui:width default:1E-3\n\
     height type:#worldUnits ui:height default:1E-3\n\
   )\n\
   rollout params &quot;SquareTube&quot;\n\
   (\n\
     spinner height &quot;Height&quot; type:#worldUnits range:[1E-3,1E9,1E-3]\n\
     spinner width &quot;Width&quot; type:#worldUnits range:[1E-3,1E9,1E-3]\n\
     spinner length &quot;Length&quot; type:#worldUnits range:[-1E9,1E9,1E-3]\n\
   )\n\
   on buildMesh do\n\
   (\n\
     if box1 == undefined do\n\
       (box1 = createInstance box; box2 = createInstance box)\n\
     box1.height = height; box2.height = height\n\
     box1.width = width; box2.width = width * 2\n\
     box1.length = length; box2.length = length * 2\n\
     mesh = box2.mesh - box1.mesh\n\
   )\n\
   tool create\n\
   (\n\
     on mousePoint click do\n\
       case click of\n\
       (\n\
         1: nodeTM.translation = gridPoint\n\
         3: #stop\n\
       )\n\
     on mouseMove click do\n\
       case click of\n\
       (\n\
         2: (width = abs gridDist.x; length = abs gridDist.y)\n\
         3: height = gridDist.z\n\
       )\n\
     )\n\
   )\n\
</code></pre></blockquote>\n\
<p>In this example, the parameters and rollouts are handled in a similar manner to the first example, but the <code>buildMesh</code> handler generates the mesh in an indirect way. The final object is in the form of a rectangular pipe, a box with a box hole through the middle. Two plug-in locals ( <code>box1</code> and <code>box2</code> ) are made to contain Box base objects whose size parameters are set according to the plug-in&apos;s parameters ( <code>box2</code> is the outer box and <code>box1</code> is the hole). The final mesh is made by boolean subtraction of <code>box1</code> from <code>box2</code> . In this case, a separate new mesh is created and assigned to the <code>mesh</code> plug-in local variable, in contrast to the first example that modified the object&apos;s original mesh directly. Either method is okay.</p>\n\
<p>The <code>createInstance()</code> method is used to directly create the box base objects. This method creates the geometry associated with the specified object, but does not create a node. This method is used since SimpleObject plug-in must not perform any scene related actions such as, creating scene nodes or building modifier stacks.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Scripted-Plug-ins.html\">Scripted Plug-ins</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Updating-Scripted-Plug-ins.html\">Updating Scripted Plug-ins</a></li>\n\
          <li><a class=\"link\" href=\"Scripted-Plug-in-Methods.html\">Scripted Plug-in Methods</a></li>\n\
          <li><a class=\"link\" href=\"Scripted-Plug-in-Clauses.html\">Scripted Plug-in Clauses</a></li>\n\
          <li><a class=\"link\" href=\"Scripted-Plug-ins-1.html\">Scripted Plug-ins</a></li>\n\
          <li><a class=\"link\" href=\"../Scripted-Mouse-Tools.html\">Scripted Mouse Tools</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";