var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-6E2727F1-BF6E-4C3D-AF3B-121B1B59DACF\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Managing Multiple Rollouts in a Scripted Utility</h1></div><p>MAXScript includes support for managing multiple rollouts in one utility, which is useful in large scripted utilities that are unwieldy in one rollout. There is one main rollout defined by the user-interface items in the utility definition. The other rollouts are defined as nested rollouts within the body of the utility definition. These other rollouts can be added to and removed from the Utilities panel under script control.</p>\n\
<p>You can define extra rollouts with the <code>rollout</code> clause inside a utility definition:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">utility foo &quot;name&quot;\n\
(\n\
local ...\n\
spinner ...\n\
...\n\
rollout baz &quot;name&quot;\n\
(\n\
local ...\n\
checkbox ..\n\
on ...\n\
)\n\
...\n\
)\n\
</code></pre><p>Such nested rollouts can contain anything that a utility definition can contain with the exception of further nested rollouts. Nested rollouts are not automatically opened or closed when the parent utility rollout opens and closes. You need to explicitly do this in the open and close handlers, or other handlers or functions in the utility.</p>\n\
<p>There are two functions for this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">addRollout &lt;rollout&gt; [ rolledUp:&lt;boolean&gt; ] removeRollout &lt;rollout&gt;</code></pre><p>The <code>rolledUp:</code> parameter on the <code>addRollout()</code> function specifies whether the rollout is added in a rolled-up state. This defaults to <code>false</code> , so the rollout is added fully opened. The nested rollouts are arranged in the order of <code>addRollout()</code> calls, so take care in sequencing these to ensure the order you want.</p>\n\
<p>The nested rollouts do not automatically get removed when the main utility rollout is closed. You must make sure they are explicitly removed, typically in the close handler for the main utility.</p>\n\
<p>To always open and close extra rollouts when the main utility is opened and closed, place <code>addRollout()</code> and <code>removeRollout()</code> calls in the main utility open and close handlers.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   on foo open do\n\
   (\n\
   ...\n\
   addRollout panel_1\n\
   addRollout panel_2 rolledUp:true\n\
   ...\n\
   )\n\
   on foo close do\n\
   (\n\
   ...\n\
   removeRollout panel_1\n\
   removeRollout panel_2\n\
   ...\n\
   )\n\
</code></pre></blockquote>\n\
<p>Calling <code>removeRollout()</code> on an already closed rollout is OK and does nothing. If during utility development, you forget to close a nested rollout, closing the entire MAXScript utility will remove any remaining open scripted rollouts.</p>\n\
<p>Because these rollout definitions are nested within a utility, all the locals, functions, and other items in the utility are visible to nested rollouts, following the standard nested scoping in MAXScript. This means you can &quot;reach out&quot; and access the utility&#x2019;s locals, user-interface control items, and functions.</p>\n\
<p>Conversely, user-interface control items and locals in nested rollouts are exposed as properties of that rollout, so code in the main utility or other rollouts can &quot;reach in&quot; and access them. In the following example, the spinner <code>bar</code> change handler looks at the state of a check box in the nested rollout <code>baz</code> :</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   utility foo &quot;name&quot;\n\
   (\n\
   local ...\n\
   local ...\n\
   spinner bar ...\n\
   ...\n\
   rollout baz &quot;name&quot;\n\
   (\n\
   local ...\n\
   checkbox baz_enable ...\n\
   on ...\n\
   )\n\
   ...\n\
   on bar changed val do\n\
   if baz.baz_enable.checked == true then...\n\
</code></pre></blockquote>\n\
<p>You can control the grouping of rollouts by supplying a <code>category:&lt;integer&gt;</code> header parameter before the opening parenthesis of the rollout body. The category numbers order the rollouts. In the following example, the rollouts are defined and added in a certain order, but displayed in another based on their category number:</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   rollout first_rollout &quot;1st Rollout&quot; category:2\n\
   (\n\
   label rights &quot;First&quot;\n\
   )\n\
   rollout second_rollout &quot;2nd Rollout&quot; category:1\n\
   (\n\
   label second &quot;Second&quot;\n\
   )\n\
   rf = newRolloutFloater &quot;Rollout Order Test&quot; 200 140\n\
   addRollout first_rollout rf\n\
   addRollout second_rollout rf\n\
   )\n\
</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Scripted-Utilities-and-Rollouts.html\">Scripted Utilities and Rollouts</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Scripted-Utilities-and-Rollouts.html\">Scripted Utilities and Rollouts</a></li>\n\
          <li><a class=\"link\" href=\"Scripted-Utility-Panels.html\">Scripted Utility Panels</a></li>\n\
          <li><a class=\"link\" href=\"Utility-Clauses.html\">Utility Clauses</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";